#!/usr/bin/perl -w
package core::config;
use core::utils;
use core::errorcodes;

use strict;
use warnings;

my $MINLOGFILESIZECONFIG = 1048576;
my $MAXLOGFILESIZECONFIG = 10485760;

# Description: Read the configuration from config file
# Params: NA
# Return: NA or die on failure
sub readconfigfile
{
    my $config_line;
    my $Name;
    my $Value;
    my $ret;

    my ($configfile, $Config) = @_;

    if (!-e $configfile)
    {
        return $core::errorcodes::MIG_SUCCESS;
    }

    $ret = core::utils::checkfile($configfile);
    if ($ret != $core::errorcodes::MIG_SUCCESS)
    {
        return $ret;
    }

    if (!open (CFGFILE, "$configfile")) 
    {
        core::logger::writeerrorconsolepremigratelogret(__FILE__, __LINE__, "Opening config file $configfile failed with $!");
        return $core::errorcodes::FILE_OPEN_FAILED;        
    }

    while (<CFGFILE>) 
    {
        my $cfgline = $_;
        if (!defined $cfgline)
        {
            core::logger::writeerrorconsolepremigratelogret(__FILE__, __LINE__, "Reading from file $configfile failed");     
            close(CFGFILE);
            return $core::errorcodes::FILE_READ_FAILED;  
        }
            
        chomp($cfgline);          # Get rid of the trailling \n
        $cfgline =~ s/^\s*//;     # Remove spaces at the start of the line
        $cfgline =~ s/\s*$//;     # Remove spaces at the end of the line
        if (($cfgline !~ /^#/) && ($cfgline ne ""))    # Ignore lines starting with # and blank lines
        {
            ($Name, $Value) = split (/=/, $cfgline);          # Split each line into name value pairs
            $$Config{$Name} = $Value;                             # Create a hash of the name value pairs
        }
    }

    close(CFGFILE);
    return $core::errorcodes::MIG_SUCCESS;
}

# Description: update the configuration with the configuration read from application.properties config file
# Params: Config map
# Return: NA or die on failure
sub updateapplicationconfiguration
{
    my (%configmap) = @_;
    if (!defined($core::teradatacore::targetfiles))
    {
        $core::teradatacore::targetfiles = $configmap{'target_files'};
    }
    
    if (!defined($configmap{'NoOfThreads'}))
    {    
        $core::teradatacore::num_of_threads = $configmap{'NoOfThreads'};
    }
    
    if ($core::teradatacore::encodingformat ne '' and $configmap{'encodingFormat'} ne '')
    {    
        $core::teradatacore::encodingformat = $configmap{'encodingFormat'};
    }    
	
	if ($core::teradatacore::exectimeout == 0 and $configmap{'Timeout'} ne '')
    {    
        $core::teradatacore::exectimeout = $configmap{'Timeout'};
		if ($core::teradatacore::exectimeout < 4 or $core::teradatacore::exectimeout > 20)
		{
			print "Invalid value specified for timeout option, continuing with default value\n";    
		}
		else
		{
			$core::teradatacore::exectimeout = 4;
		}
    }    
	
	my $defMaxFileSizeinBytes = 20 * 1024 * 1024;
	my $maxFileSizeStr = uc $configmap{'MaxFileSize'};
		
	if($maxFileSizeStr =~ m/(\d+)\s*(GB|MB|KB|B)/)
	{
		my $size = $1;
		my $byteValue = $2;
		my $fileSizeInBytes = $size;
		if( $byteValue eq 'GB')
		{
			if ($size > 1) {
				$configmap{'MaxFileSize'}=$defMaxFileSizeinBytes;
			}
			else
			{
				$fileSizeInBytes = $fileSizeInBytes * 1024 * 1024 * 1024;
				$configmap{'MaxFileSize'}=$fileSizeInBytes;
			}
		}
		elsif( !($byteValue eq "B" || $byteValue eq "KB" || $byteValue eq "MB") )
		{
			$configmap{'MaxFileSize'}=$defMaxFileSizeinBytes;
		}
		else{
			
			if( $byteValue eq 'KB')
			{
				$fileSizeInBytes = $fileSizeInBytes * 1024;
			}
			if( $byteValue eq 'MB')
			{
				$fileSizeInBytes = $fileSizeInBytes * 1024 * 1024;
			}
			
			$configmap{'MaxFileSize'}=$fileSizeInBytes;
		}
	
	}
	else
	{
		$configmap{'MaxFileSize'}=$defMaxFileSizeinBytes;
	}
	    
		$core::teradatacore::max_file_size = $configmap{'MaxFileSize'};
    return;
}    

# Description: update the configuration with the configuration read from perl config file
# Params: Config map
# Return: NA or die on failure
sub updateperlconfiguration
{
    my (%configmap) = @_;
    my $configmaxfilesize;
    my $configmaxlogfilecount;
    
    if (!defined($core::teradatacore::bteqtagname) or '' eq  $core::teradatacore::bteqtagname)
    {
        $core::teradatacore::bteqtagname = $configmap{'db-bteq-tag-name'};
    }
    if (!defined($core::teradatacore::tdsqltagname) or '' eq  $core::teradatacore::tdsqltagname)
    {
        $core::teradatacore::tdsqltagname = $configmap{'db-tdsql-tag-name'};
    }
    if (!defined($core::teradatacore::addtimingon))
    {
        $core::teradatacore::addtimingon = $configmap{'add-timing-on'};
        if ('true' ne lc($core::teradatacore::addtimingon) and 'false' ne lc($core::teradatacore::addtimingon) and $core::teradatacore::addtimingon ne '')
        {
            print "Invalid value specified for add-timing-on option, continuing with default value\n";
            $core::teradatacore::addtimingon = '';
        }

        if ('false' eq  lc($core::teradatacore::addtimingon))
        {
            $core::teradatacore::addtimingon = '';
        }
    }
    
    if (!defined($core::teradatacore::migrateexecutecontents))
    {
        $core::teradatacore::migrateexecutecontents = $configmap{'migrate-executequery'};
        if ('true' ne lc($core::teradatacore::migrateexecutecontents) and 'false' ne lc($core::teradatacore::migrateexecutecontents) and $core::teradatacore::migrateexecutecontents ne '')
        {
            print "Invalid value specified for migrate-executequery option, continuing with default value\n";
            $core::teradatacore::migrateexecutecontents = '';
        }

        if ('false' eq  lc($core::teradatacore::migrateexecutecontents))
        {
            $core::teradatacore::migrateexecutecontents = '';
        }    
    }
    
    if (!defined($core::teradatacore::removeintermediatefiles))
    {
        $core::teradatacore::removeintermediatefiles = $configmap{'remove-intermediate-files'};
        if ('true' ne lc($core::teradatacore::removeintermediatefiles) and 'false' ne lc($core::teradatacore::removeintermediatefiles) and $core::teradatacore::removeintermediatefiles ne '')
        {
            print "Invalid value specified for remove-intermediate-files option, continuing with default value\n";
            $core::teradatacore::removeintermediatefiles = '';
        }
        if ('false' eq  lc($core::teradatacore::removeintermediatefiles))
        {
            $core::teradatacore::removeintermediatefiles = '';
        }
    }
    if (!defined($core::teradatacore::migratevariables))
    {
        $core::teradatacore::migratevariables = $configmap{'migrate-variables'};
        if ('true' ne lc($core::teradatacore::migratevariables) and 'false' ne lc($core::teradatacore::migratevariables) and $core::teradatacore::migratevariables ne '')
        {
            print "Invalid value specified for migrate-variables option, continuing with default value\n";
            $core::teradatacore::migratevariables = 'true';
        }
        if ('false' eq  lc($core::teradatacore::migratevariables))
        {
            $core::teradatacore::migratevariables = '';
        }
    }
    
    if (!defined($core::logger::logginglevel))
    {
        $core::logger::logginglevel = $configmap{'logging-level'};
    }
    
    $configmaxfilesize = $configmap{'log-file-size'};
    if (defined($configmaxfilesize))
    {
        if ($configmaxfilesize !~ /(\d*)MB/)
        {
            print "Invalid value specified for log-file-size option, continuing with default value\n";
        }
        else 
        {
			$configmaxfilesize =~ /(\d*)MB/;
			my $configintsize = $1;
			
			# convert specified value from MB to B
			$configintsize = $configintsize * 1024 * 1024; 
			if ($configintsize < $MINLOGFILESIZECONFIG or $configintsize > $MAXLOGFILESIZECONFIG)
			{
				print "Invalid value specified for log-file-size option, continuing with default value\n";    
			}
			else
			{
				$core::teradatacore::maxlogfilesize = $configintsize;
			}
        }
    }
    
    $configmaxlogfilecount = $configmap{'log-file-count'};
    if (defined($configmaxlogfilecount))
    {
        if ($configmaxlogfilecount < 3 or $configmaxlogfilecount > 10)
        {
            print "Invalid value specified for log-file-count option, continuing with default value\n";    
        }
        else
        {
            $core::teradatacore::maxlogfilecount = $configmaxlogfilecount;
        }
    }
    
    return;
}

return 1;
