#!/usr/bin/perl -w
package core::logger;
use POSIX qw(strftime);

use strict;
use warnings;

use File::stat;
use File::Path;
use File::Basename;
use File::Copy;
use File::Find;
use threads;
use threads::shared;

use core::errorcodes;

my $logmessagelock:shared;
our $loggingpath;
our $logginglevel;

my $logfilename;
our $logfileinitiated = 0;
our $currentlogfilesize:shared = 0;
our $maxlogfilecount = 5;
our $maxlogfilesize = 5242880; #5 MB (5*1024*1024)
our $LOGHANDLE:shared;

my $NEWLINE = "\n";
my $SPACE = " ";
my $COLON = ":";
my $systemversion = $^O;

no warnings "uninitialized";

# Description: Initialize the log for perl migration
# Params: NA
# Return: NA or die on failure
sub initlog
{
    my $ret;
    if (!-e $loggingpath)
    {    
        eval
        {
            if (!mkpath($loggingpath, 0, 0700))
            {
                writeerrorconsolelogret(__FILE__, __LINE__, "Creating directory: $loggingpath failed");      
                return $core::errorcodes::DIR_CREATE_FAILED;
            }

            if (!($core::teradata::systemversion !~ /Microsoft/ && $core::teradata::systemversion !~ /MSW/))
			{ 
				my $chmodcmd = sprintf "icacls %s \/grant:r \*%s:(F,MA) \/inheritance:r", $loggingpath, $core::teradatacore::usersid;
				my $chmodcmdresult =`$chmodcmd`;
			}
        };
        if ($@) 
        {
            writeerrorconsolelogret(__FILE__, __LINE__, "Creating directory: $loggingpath failed");       
            return $core::errorcodes::DIR_CREATE_FAILED;
        }        
    }
    else
    {        
        $ret = core::utils::checkdir($loggingpath);
        if ($ret != $core::errorcodes::MIG_SUCCESS)
        {
            return $ret;
        }
    }
    
    checkremoveoldlogfiles();
    
    $logfilename = $loggingpath.$core::teradatacore::directoryseparator."perldsc.log";
    if (!open(LOGHANDLE, ">>$logfilename"))
    {
        writeerrorconsolelogret(__FILE__, __LINE__, "Opening of file $logfilename failed with $!");    
        return $core::errorcodes::FILE_OPEN_FAILED;
    }
    
    $currentlogfilesize = -s $logfilename;

	if ($core::teradata::systemversion !~ /Microsoft/ && $core::teradata::systemversion !~ /MSW/)
	{ 
		if (!chmod(0600, $logfilename))
		{
			writeerrorconsolelogret(__FILE__, __LINE__, "Changing permissions for file $logfilename failed with $!");  
			close(LOGHANDLE);        
			return $core::errorcodes::CHMOD_FAILED;
		}
	}
	else
	{ 
		my $chmodcmd = sprintf "icacls %s \/grant:r \*%s:(F,WDAC) \/inheritance:r", $logfilename, $core::teradatacore::usersid;
		my $chmodcmdresult =`$chmodcmd`;
	}
    
    
    $logfileinitiated = 1;
    return $core::errorcodes::MIG_SUCCESS;
}

# Description: close the log  file
# Params: NA
# Return: NA or die on failure
sub destroylog
{
    if ($logfileinitiated == 1)
    {
        close(LOGHANDLE);
        $logfileinitiated = 0;
    }

    return;
}

# Description: Check and remove old log files
# Params: NA
# Return: NA or die on failure
sub checkremoveoldlogfiles
{
    my @logfilelist;
    my $removecount = 0;
    my $listcount = 0;
    opendir(SUB, $loggingpath) or writeerrorconsoleandlog(__FILE__, __LINE__, "opendir $loggingpath failed");
    my @tmpfilelist = readdir(SUB);
    closedir(SUB);        

    my $file_to_act;
    foreach $file_to_act (@tmpfilelist)
    {
        if ($file_to_act =~ /\.$/)
        {
            next;
        }
        else
        {
            if (-f "$loggingpath$core::teradatacore::directoryseparator$file_to_act")
            {    
                if  ($file_to_act =~ m/perldsc_(.*?)\.log/)
                {                
                    push(@logfilelist, $file_to_act);
                }
            }                
        }
    }
    
    $listcount = @logfilelist;
    if ($listcount >= $maxlogfilecount)
    {
        $removecount = ($listcount - $maxlogfilecount) + 1;
    }
    else
    {
        return;
    }
    
    # rsort the list
    @logfilelist = sort
    {
        $a =~ m/perldsc_(\d{4})-(\d{2})-(\d{2})_(\d{2})_(\d{2})_(\d{2})\.log/;
        my $ayear = $1;
        my $amonth = $2;
        my $aday = $3;
        my $ahour = $4;
        my $aminute = $5;
        my $asecond = $6;
        
        $b =~ m/perldsc_(\d{4})-(\d{2})-(\d{2})_(\d{2})_(\d{2})_(\d{2})\.log/;
        my $byear = $1;
        my $bmonth = $2;
        my $bday = $3;
        my $bhour = $4;
        my $bminute = $5;
        my $bsecond = $6;
        
        # Compare year           
        my $ret = core::utils->compare($ayear, $byear);        
        if ($ret != 0)
        {
            return $ret;
        }
        
        # Compare month
        $ret = core::utils->compare($amonth, $bmonth);
        if ($ret != 0)
        {
            return $ret;                
        }        
        
        # Compare day
        $ret = core::utils->compare($aday, $bday);
        if ($ret != 0)
        {
            return $ret;            
        }    
        
        # Compare hour
        $ret = core::utils->compare($ahour, $bhour);
        if ($ret != 0)
        {
            return $ret;            
        }
                
        # Compare min
        $ret = core::utils->compare($aminute, $bminute);
        if ($ret != 0)
        {
            return $ret;
        }
        
        # Compare second
        if($asecond > $bsecond) 
        {
            return 1;
        }
        elsif($bsecond > $asecond) 
        {
            return -1;
        }

        return 0;            
    } @logfilelist;
    
    foreach $file_to_act (sort @logfilelist)
    {
        if ($removecount > 0)
        {        
            print "Removing log file :$loggingpath$core::teradatacore::directoryseparator$file_to_act\n";
            unlink("$loggingpath$core::teradatacore::directoryseparator$file_to_act");
            $removecount--;
        }
    }    
}

# Description: Switch the log  file
# Params: NA
# Return: NA or die on failure
sub switchlogfile
{
    my $datestring = strftime "%F_%H_%M_%S", localtime;
    my $newlogfilename = "$loggingpath$core::teradatacore::directoryseparator"."perldsc_$datestring.log";
    destroylog();
    
    move($logfilename, $newlogfilename) or die "Renaming log file failed\n"; 
    
    checkremoveoldlogfiles();
    
    open(LOGHANDLE, ">$logfilename") or die "Opening of file $logfilename failed with $!\n";    
    $currentlogfilesize = -s $logfilename;

    if ($core::teradata::systemversion !~ /Microsoft/ && $core::teradata::systemversion !~ /MSW/)
	{ 
		if (!chmod(0600, $logfilename))
		{    
			close(LOGHANDLE);
			die "changing permissions for file $logfilename\n";         
		}
	}
	else
	{ 
		my $chmodcmd = sprintf "icacls %s \/grant:r \*%s:(F,WDAC) \/inheritance:r", $logfilename, $core::teradatacore::usersid;
		my $chmodcmdresult =`$chmodcmd`;
	}
    
    $logfileinitiated = 1;    
}

# Description: Write the log message
# Params: NA
# Return: NA or die on failure
sub writelogmsg
{
	lock $logmessagelock;
    my ($filename, $linenumber, $logginglevel, $inputmsg) = @_;
    my $formattedmsg;
    if ($logfileinitiated == 0)
    {
        return;
    }
    
    my $datestring = strftime "%F %H:%M:%S", localtime;
    my $basefile = fileparse($filename);
    if ($linenumber != 0)
    {
        $formattedmsg = $datestring.$SPACE.$logginglevel.$SPACE.$basefile.$COLON.$linenumber.$SPACE.$inputmsg.$NEWLINE;
    }
    else
    {
        $formattedmsg = $inputmsg.$NEWLINE;
    }
    
    my $formatsize = length($formattedmsg);    
    $currentlogfilesize = $currentlogfilesize + length($formattedmsg);
    
    if ($currentlogfilesize > $maxlogfilesize)
    {
        switchlogfile();
    }
    
    print LOGHANDLE "$formattedmsg";
    return;
}

# Description: Write info log message
# Params: NA
# Return: NA or die on failure
sub writeinfologmsg
{
	if (defined($logginglevel) && $logginglevel ne "INFO")
    {
        return;
    }

    my ($filename, $linenumber, $inputmsg) = @_;
    writelogmsg($filename, $linenumber, "INFO", $inputmsg);
    return;
}

# Description: Write warning log message
# Params: NA
# Return: NA or die on failure
sub writewarninglogmsg
{
    if (defined($logginglevel) && $logginglevel ne "ERROR")
    {
        return;
    }
    
    my ($filename, $linenumber, $inputmsg) = @_;
    writelogmsg($filename, $linenumber, "WARNING", $inputmsg);
    return;
}

# Description: Write errog log message
# Params: NA
# Return: NA or die on failure
sub writeerrorlogmsg
{
    my ($filename, $linenumber, $inputmsg) = @_;
    writelogmsg($filename, $linenumber, "ERROR", $inputmsg);
    return;
}

# Description: Write info message to log and console
# Params: NA
# Return: NA or die on failure
sub writeinfoconsoleandlog
{
	if (defined($logginglevel) && $logginglevel ne "INFO")
    {
        return;
    }

    my ($filename, $linenumber, $inputmsg) = @_;
    writelogmsg($filename, $linenumber, "INFO", $inputmsg);
    print "$inputmsg\n";
    return;
}

# Description: Write warning message to log and console
# Params: NA
# Return: NA or die on failure
sub writewarningconsoleandlog
{
	if (defined($logginglevel) && $logginglevel ne "ERROR")
    {
        return;
    }
    
    my ($filename, $linenumber, $inputmsg) = @_;
    writelogmsg($filename, $linenumber, "WARNING", $inputmsg);
    print "$inputmsg\n";
    return;
}

# Description: Write errog message to log and console
# Params: NA
# Return: NA or die on failure
sub writeerrorconsoleandlog
{
    my ($filename, $linenumber, $inputmsg) = @_;
    writelogmsg($filename, $linenumber, "ERROR", $inputmsg);
    die "$inputmsg\n";
}

# Description: Write errog message to log, console and return
# Params: NA
# Return: NA or die on failure
sub writeerrorconsolelogret
{
    my ($filename, $linenumber, $inputmsg) = @_;    
    writelogmsg($filename, $linenumber, "ERROR", $inputmsg);    
    return;
}

# Description: Write errog message to log, console and return
# Params: NA
# Return: NA or die on failure
sub writeerrorconsolepremigratelogret
{
    my ($filename, $linenumber, $inputmsg) = @_;    
    writelogmsg($filename, $linenumber, "ERROR", $inputmsg);    
    
    if ($core::teradatadacore::premigrate == 1)
    {
        print "$inputmsg\n";
    }
    
    return;
}

# Description: Write errog message to log, console and return
# Params: NA
# Return: NA or die on failure
sub writeerrorconslogret
{
    my ($filename, $linenumber, $inputmsg) = @_;    
    writelogmsg($filename, $linenumber, "ERROR", $inputmsg);    
    print "$inputmsg\n";
    return;
}

# Description: write msg to console
# Params: inputmsg
# Return: NA 
sub writeinfoconsole
{
    if ($logginglevel ne "INFO")
    {
        return;
    }
    my ($inputmsg) = @_;
    print "$inputmsg";
    return;
}

return 1;
