#!/usr/bin/perl -w
package core::teradatacore;

use core::help;
use core::version;
use core::logger;
use core::utils;
use core::progress;
use core::config;
use core::errorcodes;

use strict;
use warnings;

use threads;
use threads::shared;

use Getopt::Long qw(:config no_ignore_case);
use File::stat;
use File::Path;
use File::Basename;
use File::Copy;
use File::Find;

no warnings "uninitialized";

# option variables
our $bteqtagname;
# Regex formatted bteq list
our $bteqtaglist;
our $tdsqltagname;
# Regex formatted sql lang list
our $tdsqltaglist;
our $location;
our $migrationtoolpath = 'GaussTeradataDSC-V4.7';
our $addtimingon;
our $removeintermediatefiles;
our $migratevariables;
our $targetfiles;
our $targetdb;
our $outputfolder;
our $migrateexecutecontents;
our $exectimeout = 0;
our $help     = 0;
our $version  = 0;
our $nopl     = 0;

our @fileList :shared;
# Define the number of threads
our $num_of_threads = 1;
our $encodingformat = "";
our $max_file_size = 20*1024*1024;

our $perlfilecount:shared = 0;
our $allperlfilecount:shared = 0;
our $totalperlfilecount:shared = 0;
our $migratedperlfilescount:shared = 0;

# global variables
our $lockfilepath;
our $lockfilecreated = 0;

my %varnamedetails :shared;
my %varcountdetails :shared;
our %sqlfilecreateddetails:shared;
our %encodingfiledetails:shared;
my %executequerydetails :shared;
my %executequerycountdetails :shared;

our $commandexecutioncompleted:shared;
our $threadExit:shared = 0;

our $extractionfailcount:shared = 0;
our $mergefailcount:shared = 0;
our $extractiondirfailcount:shared = 0;
our $mergedirfailcount:shared = 0;
our $premigrate = 1;
our $systemversion = "";
our $directoryseparator = "";

#ERROR Codes

# command to get security identifier for windows
our $usersid:shared;
my $perlfilecountlock:shared;
my $chmodcommand;
my $chmodcmdresult;
my %appconfig;

#Error information
my %ERRORDETAILS = (
'0', 'Success',
'1', 'File open failed',
'2', 'File read failed',
'3', 'File write failed',
'4', 'File seek failed',
'5', 'Chmod of file failed',
'6', 'Open directory failed',
'7', 'Invalid permissions',
'8', 'Invalid argument',
'9', 'Directory creation failed',
'10', 'Removing directory failed',
'11', 'Command execution failed'
);

#options for sql to gauss
GetOptions(
    'input-folder|i=s'               => \$location,
    'db-bteq-tag-name|d=s'           => \$bteqtagname,    
    'db-sql-tag-name|s=s'            => \$tdsqltagname, 
    'dsc-path|m=s'        		     => \$migrationtoolpath,
    'add-timing-on|a!'               => \$addtimingon,
    'remove-intermediate-files|r!'   => \$removeintermediatefiles,
    'migrate-variables|v!'           => \$migratevariables,
    'migrate-executequery|e!'        => \$migrateexecutecontents,    
    'target_files|t=s'               => \$targetfiles,
    'target-db|g=s'		             => \$targetdb,    
    'output-folder|o=s'              => \$outputfolder,
    'log-folder|l=s'                 => \$core::logger::loggingpath,
    'help|H!'                        => \$help,
    'version|V!'                     => \$version,
    'thread-count|c=i'               => \$num_of_threads,
    'encoding-format|f=s'            => \$encodingformat
) or die "Incorrect usage!\n";

#handle backward compatibility
if ($location eq '')
{
    $location = $ARGV[0];
}

if (!defined($targetdb) or '' eq $targetdb) 
{
	$targetdb = "gauss200";
}

sub getWinSid
{
	if ($systemversion =~ /Microsoft/ || $systemversion =~ /MSW/)
	{
		my $getsid = "wmic useraccount where name='%username%' get sid";
		my $commandresult = `$getsid`;
		
		my @lines = split /\n/, $commandresult;
		foreach my $line (@lines) 
		{            
			if ($line =~ m/S-/)
			{
				$line =~ m/\s*(S-.*)\s*/;
				$usersid = $1;
				$usersid =~ s/^\s+|\s+$//g;
				return;
			}		
		}
	}
}

# Description: Initialize the threads
# Params:
# Return: NA 
sub initThreads
{
    my @initThreads;

    for(my $i = 1;$i<=$num_of_threads;$i++)
    {
        push(@initThreads,$i);
    }

    return @initThreads;
}

# Description: Add the elements to the list
# Params:
# Return: NA 
sub addelementtolist
{
    my ($current_loc) = @_;
    push  @fileList,$current_loc;
    return;
}

# Description: Retrieve an element from the list
# Params:
# Return: NA 
sub getelementfromlist
{
    lock @fileList;
    my $file = pop @fileList;
    return $file;
}

# Description: Execute migration command
# Params: migrationexecoutput
# Return: NA 
sub executemigrationcommand
{
    my $migrationexecoutput = shift;
    my $ret;
    
    core::logger::writeinfoconsoleandlog(__FILE__, __LINE__, "Stage 2/3: Converting SQL contents.");
    #Execute DSC on the generated scripts
    my $migrationcommand;
    if (-f $location)
    {
        my $basefilepath = dirname($location);
        $migrationcommand = join "", "java -jar ", $migrationtoolpath, $directoryseparator,"bin", $directoryseparator, "dsctool.jar --source-db teradata --input-folder ", $basefilepath, " --output-folder ", $outputfolder, " --log-folder ", $core::logger::loggingpath, " --target-db ", $targetdb;
    }
    else
    {
        $migrationcommand = join "", "java -jar ", $migrationtoolpath, $directoryseparator,"bin", $directoryseparator, "dsctool.jar --source-db teradata --input-folder ", $location, " --output-folder ", $outputfolder, " --log-folder ", $core::logger::loggingpath, " --target-db ", $targetdb;    
    }

    my $displaythread = threads->create(\&core::progress::displaymigrationprogress);

    # Maintaining backward compatibility
    # Update the migration command here if you do not wish to specify the --migration-tool-path option
    $ret = core::utils::execcommand($migrationcommand, $$migrationexecoutput, 1);
    if ($ret != $core::errorcodes::MIG_SUCCESS)
    {
        $displaythread->join();        
        return $ret;
    }

    $displaythread->join();    
    core::progress::clearcurrentlineprintlog(__FILE__, __LINE__, "DSC completed.");
    return $core::errorcodes::MIG_SUCCESS;
}

# Description: Extract the sql contents
# Params:
# Return: NA 
sub extractsqlcontents
{
    # Extracting the BTEQ tags from the config value
    if ($bteqtagname ne '')
    {
        my @bteqnameList = split(',', $bteqtagname);

		# trim the space from the array item
        map { s/^\s+|\s+$//g; } @bteqnameList;

		# Remove the empy strings in the array
		@bteqnameList = grep /\S/, @bteqnameList;

		# join the array elements with |
        my $taglist = join("|", @bteqnameList);
        $bteqtaglist = "($taglist)";
    }

    # Extracting the SQL_LANG tags from the config value
    if ($tdsqltagname ne '')
    {
        my @sqlnameList = split(',', $tdsqltagname);

		# trim the space from the array item
        map { s/^\s+|\s+$//g; } @sqlnameList;

		# Remove the empy strings in the array
		@sqlnameList = grep /\S/, @sqlnameList;

		# join the array elements with |
        my $taglist = join("|", @sqlnameList );
        $tdsqltaglist = "($taglist)";
    }

    @fileList = sort @fileList;
    my $currcount = 0;
    my $threadexitcount = 0;
    
    #use the initThreads subroutine to create an array of threads.
    my @threads = initThreads();
    
    my $displaythread = threads->create(\&core::progress::displayperlprogress, "Stage1");

    #Loop through the array:
    foreach(@threads)
    {
        $_ = threads->create(\&extractsqlcontentsthread);
    }    
        
    # This tells the main program to keep running until all threads have finished.
    foreach(@threads)
    {
        $_->join();       
    }    

    $displaythread->join();        
    return $threadExit;
}

# Description: Merget the contents from the thread
# Params:
# Return: NA 
sub mergesqlcontents
{
    @fileList = sort @fileList;
    my $threadexitcount;
    my $currcount;

    # use the initThreads subroutine to create an array of threads.
    my @threads = initThreads();
    
    my $displaythread=threads->create(\&core::progress::displayperlprogress, "Stage3");

    # Loop through the array:
    foreach(@threads)
    {       
        $_ = threads->create(\&mergesqlcontentsthread);
    }        
    
    # This tells the main program to keep running until all threads have finished.
    foreach(@threads)
    {
        $_->join();       
    }    

    core::logger::writeinfologmsg(__FILE__, __LINE__, "Wait for display thread");
    $displaythread->join();    
    return $threadExit;
}

# Description: Recursively reads the list of pl file present and calls act_file or act_filec
# Params: input location specified by user, preprocess or postprocess
# Return: NA or die on failure
sub readsub
{
    my $file_t  = shift;
    my $postprocess  = shift;
    my $faildircount  = shift;
    
    my $ret;
    if (-f $file_t)
    {
        if ($file_t !~ m/.*\.pl$/)
        {
            $nopl = 1;
            return $core::errorcodes::MIG_SUCCESS;
        }

        $ret = core::utils::checkfile($file_t);
        if ($ret != $core::errorcodes::MIG_SUCCESS)
        {
            return $ret;
        }
        
        if ($postprocess == 0)
        {
            core::logger::writeinfoconsoleandlog(__FILE__, __LINE__, "Extracting SQL contents from perl files started");
            $ret = act_file($file_t);
            incrementmigratedperlfilescount();
            if ($ret != $core::errorcodes::MIG_SUCCESS)
            {
                return $ret
            }
        }
        else
        {
            $ret = act_filec($file_t);
            incrementmigratedperlfilescount();    
            if ($ret != $core::errorcodes::MIG_SUCCESS)
            {
                return $ret
            }
        }
    }
    if ( -d $file_t )
    {    
        $ret = core::utils::checkdir($file_t);
        if ($ret != $core::errorcodes::MIG_SUCCESS)
        {
            return $ret;
        }
                
        if (!opendir(SUB, $file_t))
        {
            core::logger::writeerrorconsolepremigratelogret(__FILE__, __LINE__, "opendir $file_t failed with $!");
            return $core::errorcodes::OPEN_DIR_FAILED;
        }
            
        my @tmpfilelist = readdir(SUB);
        closedir(SUB);    

        my $file_to_act;
        foreach $file_to_act (sort @tmpfilelist)
        {
            if ($file_to_act =~ /\.$/)
            {
                next;
            }
            else
            {
                if (-f "$file_t$directoryseparator$file_to_act")
                {    
                    if  ($file_to_act =~ m/\.*\.pl$/)
                    {
                        addelementtolist("$file_t$directoryseparator$file_to_act");
                    }
                }
                if (-d "$file_t$directoryseparator$file_to_act")
                {
                    $ret = readsub("$file_t$directoryseparator$file_to_act", $postprocess, $faildircount);
                    if ($ret != $core::errorcodes::MIG_SUCCESS)
                    {
                        $$faildircount = $$faildircount + 1;
                    }                    
                }                
            }
        }
    }
    
    return $core::errorcodes::MIG_SUCCESS;
}

# Description: to increment migrated perl file counter
# Params:
# Return: NA
sub incrementmigratedperlfilescount
{
    lock $perlfilecountlock;
    $migratedperlfilescount++;
    return;
}

# Description: Extract variable contents from the perl file to the sql file
# Params:
# Return: NA 
sub extractVariableContents
{
    my $FISQL_VAR = shift;
    my $file_in = shift;
    my $FOSQL_VAR = shift;
    my $file_out = shift;    
    my $varlinenumber = shift;
    my $varname = shift;
    my $varstartlinenumber = shift;
    my $line = shift;  
    my $varcount = shift;        
    my $startlinenumber = shift;
    my $endlinenumber = shift;    
    my $actualvariablename = shift;        
    my $varfound = 0;
    my $sqlcontents;
    my $varendlinenumber = 0;
        
    if ($migratevariables)
    {
        # Generate sql file for variable
        while (<FISQL_VAR>)
        {
            my $sqlvarline = $_;
            if (!defined $sqlvarline)
            {
                core::logger::writeerrorconsolelogret(__FILE__, __LINE__, "Reading from file $file_in failed");
                return $core::errorcodes::FILE_READ_FAILED;
            }
            
            $$varlinenumber = $$varlinenumber + 1;
            # Identify the starting location where variable is found
            # to avoid my $sql_IfHY=$dbh->prepare($sql_IfHY); $sql_IfHY->execute(); my $sql_IfHY1=$sql_IfHY->fetchrow();
            if ($sqlvarline =~ m/\Q$$varname\E\s*=\s*\"/i and $sqlvarline !~ m/^\s*#/i)
            {
                $$varstartlinenumber = $$varlinenumber;
                # For variables present in single line
                if ($sqlvarline =~ m/(.*?)"\s*;\s*$/)
                {    
                    # to handle "sql string". 
                    $sqlvarline =~ /(.*?)=\s*"(.*?)"\s*;\s*$/;
                    $$actualvariablename = $1;
                    $sqlcontents = $2;    
                    $varfound = 1;        
                    $varendlinenumber = $$varlinenumber;
                    next;
                }   
                
                # to handle "sql string". 
                $sqlvarline =~ /(.*?)=\s*"(.*?)"?\s*\.?\s*;?$/s;
                $$actualvariablename = $1;
                $sqlcontents = $2;    
                
                my $tmpvarline;
                while ($tmpvarline = <FISQL_VAR>)
                {                
                    if (!defined $tmpvarline)
                    {
                        core::logger::writeerrorconsolelogret(__FILE__, __LINE__, "Reading from file $file_in  failed");            
                        return $core::errorcodes::FILE_READ_FAILED;
                    }                
                    
                    $sqlvarline = $tmpvarline;
                    $$varlinenumber = $$varlinenumber + 1;
                    if ($tmpvarline !~ m/\Q$$line/ and $tmpvarline !~ m/(.*?)";\s*$/)
                    {                            
                        # to handle ."sql string". 
                        $tmpvarline =~ /^\s*\.?"?(.*?)"?\s*\.?\s*$/s;
                        my $extractedline = $1;
                        
                        $sqlcontents = $sqlcontents." ".$extractedline;
                    }
                    else
                    {
                        $varfound = 1;                        
                        # get the current line if ending with ;
                        if ($tmpvarline =~ m/(.*?)";\s*$/)
                        {
                            # to handle ."sql string". 
                            $tmpvarline =~ /^\s*\.?"?(.*?)"?\s*;/s;                                    
                            my $extractedline = $1;
                        
                            $sqlcontents = $sqlcontents." ".$extractedline;                        
                        }      
                        
                        $varendlinenumber = $$varlinenumber;
                        last;
                    }
                }                    
            }
            
            # Dont break from the loop, this is to handle cases where the variable will be declared and then defined like:
            # $sqlStr = ""£»
            # $sqlStr = "select * from t1;";
            if ($sqlvarline =~ m/\Q$$line/)
            {
                last;
            }
        }         
    }

    # variable found;
    if ($varfound == 1)
    {
        $$startlinenumber = $$varstartlinenumber;
        $$endlinenumber =  $varendlinenumber;        
        $$varcount = $$varcount + 1;                                    
        
        #write variable to new file
        print FOSQL_VAR "$$varname;\n";
        if ($? != 0)
        {
            core::logger::writeerrorconsolelogret(__FILE__, __LINE__, "Writing to file $file_out  failed");            
            return $core::errorcodes::FILE_WRITE_FAILED;
        }
                                        
        # add ; if the query does not have ;
        if ($sqlcontents =~ m/;\s*$/m)
        {
            print FOSQL_VAR "$sqlcontents\n";    
        }
        else
        {
            print FOSQL_VAR "$sqlcontents;\n";
        }

        # check for return value from print and handle print failure
        if ($? != 0)
        {
            core::logger::writeerrorconsolelogret(__FILE__, __LINE__, "Writing to file $file_out failed");            
            return $core::errorcodes::FILE_WRITE_FAILED;
        }                
    }    
    
    return $core::errorcodes::MIG_SUCCESS;
}

# Description: Extract sql contents present within sql_lang tag from the perl file to the sql file
# Params:
# Return: NA
sub extractSqlFromSQLLANGTag
{  
    my $FI = shift;
    my $line = shift;
    my $preparelinenumber = shift;
    my $filecontentswritten = shift;
    my $file_in = shift;
    my $file_out = shift;
    
    my $commentstarted = 0;
    my $tmp_line;
    my $next_line;  
    my $seekpos = 0;
    my $semicolonpresent = 0;
    
    if ($$line =~ m/\s*\w*\s*\$\w+\s*=\s*<\s*<\s*$tdsqltaglist\s*;?/i )
    {
        
        while ($tmp_line = <FI>)
        {              
            if (!defined $tmp_line)
            {
                core::logger::writeerrorconsolelogret(__FILE__, __LINE__, "Reading from file $file_in failed");
                return $core::errorcodes::FILE_READ_FAILED;
            }
            
            $$preparelinenumber = $$preparelinenumber + 1;
            
            if ($semicolonpresent == 0)
            {               
                if($tmp_line =~ m/^\s*$tdsqltaglist/i and $semicolonpresent == 0)
                {
                    #$$line = $$line.";";
                    $semicolonpresent = 1;
                }
                
                if($tmp_line =~ m/\;/)
                {
                    $semicolonpresent = 1;
                }                
            }
            
            # No need to skip contents within comments
            if ($commentstarted)
            {
                $tmp_line =~ s/^\s*$tdsqltagname/$tdsqltagname;/mgi;
                $$line = $$line.$tmp_line;
            }
            elsif ($tmp_line !~ m/^\s*\./m)
            {
                # $tmp_line =~ s/^\s*$tdsqltagname/$tdsqltagname;/mgi;
                $$line = $$line.$tmp_line;
            }
			elsif ($tmp_line =~ m/^\s*\./m)
			{
				if ($tmp_line =~ m/\s*\.(IF|GOTO|QUIT|RETURN|LOGOFF|RUN|EXPORT\s+FILE)/mi)
				{
					$$line = $$line.$tmp_line;
				}
			}
            
            # check if it starts with comment /*, comment started
            if ($tmp_line =~ m/^\/\*/)
            {
                $commentstarted = 1;
            }
            
            # check if it ends with comment */, comment finished
            if ($tmp_line =~ m/\*\/$/)
            {
                $commentstarted = 0;
            }
            
            
            # check if sql_lang tag is finished
            if ($tmp_line =~ m/^\s*$tdsqltaglist\s*\;?/i)
            {  
                $tmp_line = "$tmp_line;";
                last;
            }
        } 
    
        $commentstarted = 0;
        print FO $$line;
        if ($? != 0)
        {
            core::logger::writeerrorconsolelogret(__FILE__, __LINE__, "Writing to file $file_out failed");            
            return $core::errorcodes::FILE_WRITE_FAILED;
        }    
        
        $$filecontentswritten = 1;
    }  
    return $core::errorcodes::MIG_SUCCESS;
}

# Description: Extract sql contents present within bteq tag from the perl file to the sql file
# Params:
# Return: NA
sub extractSqlFromBTEQTag
{
    my $FI = shift;
    my $line = shift;
    my $preparelinenumber = shift;
    my $filecontentswritten = shift;
    my $file_in = shift;
    my $file_out = shift;    
    my $tmp_line = "";
    my $commentstarted = 0;
    my @gotolabels;
	
    if ($$line =~ m/$bteqtaglist\s*<<\s*ENDOFINPUT/i )
    {
        while ($tmp_line !~ m/\s*close\s*\(\s*$bteqtaglist\s*\);/i and $tmp_line = <FI>)
        {
            if (!defined $tmp_line)
            {
                core::logger::writeerrorconsolelogret(__FILE__, __LINE__, "Reading from file $file_in failed");            
                return $core::errorcodes::FILE_READ_FAILED;
            }    
            
            $$preparelinenumber = $$preparelinenumber + 1;
            
            # No need to skip contents within comments
            if ($commentstarted == 1)
            {
                $$line = $$line.$tmp_line;
            }	
            elsif ($tmp_line !~ m/^\s*\./m and $tmp_line !~ m/^\$\{LOGON_STR/m)
            {
                $$line = $$line.$tmp_line;
            }
			elsif ($tmp_line =~ m/^\s*\./m and $tmp_line =~ m/.*?ACTIVITYCOUNT.*?GOTO\s*\w*/m)
			{
				my $gotoline = $tmp_line;
				$gotoline =~ /.*?ACTIVITYCOUNT.*?GOTO\s*(\w*)/;
				push (@gotolabels, $1);
				$$line = $$line.$tmp_line;
				#my $label = $1;
				#print $label;
			}
			elsif ($tmp_line =~ m/^\s*\./m)
			{
				if ($tmp_line =~ m/\s*\.(IF|GOTO|QUIT|RETURN|LOGOFF|RUN|EXPORT\s+FILE)/mi)
				{
					$$line = $$line.$tmp_line;
				}
				else 
				{
					foreach my $label (@gotolabels)
					{
						if ($tmp_line =~ m/\Q$label/m)
						{
							$$line = $$line.$tmp_line;
							last;
						}
					}
				}
			}
            
            # check if it starts with comment /*, comment started
            if ($tmp_line =~ m/^\s*\/\*/)
            {
                $commentstarted = 1;
            }
            
            # check if it ends with comment */, comment finished
            if ($tmp_line =~ m/\*\/\s*$/)
            {
                $commentstarted = 0;
            }
        }
    
        $commentstarted = 0;
        print FO $$line;
        if ($? != 0)
        {
            core::logger::writeerrorconsolelogret(__FILE__, __LINE__, "Writing to file $file_out failed");            
            return $core::errorcodes::FILE_WRITE_FAILED;
        }    
        
        $$filecontentswritten = 1;
    }
    
    return;
}

# Description: Update the execute query details to the shared map
# Params:
# Return: NA
sub updateExecuteQueryMap
{
    my $varname = shift;
    my $sqlcontents = shift;
    my $execqueryvarinit = shift;
    my $file_in = shift;
    my $execquerycount = shift;
    my $startlinenumber = shift;
    my $endlinenumber = shift;
    my $fileout = shift;
    
    #write variable to new file
    print FOEXECQUERY "/* $varname */\n";
    if ($? != 0)
    {
        core::logger::writeerrorconsolelogret(__FILE__, __LINE__, "Writing to file $fileout failed");            
        return $core::errorcodes::FILE_WRITE_FAILED;
    }    
        
    # add ; if the query does not have ;
    if ($sqlcontents =~ m/;\s*$/m)
    {
        print FOEXECQUERY "$sqlcontents\n";
    }
    else
    {
        print FOEXECQUERY "$sqlcontents;\n";
    }

    # check for file write failure
    if ($? != 0)
    {
        core::logger::writeerrorconsolelogret(__FILE__, __LINE__, "Writing to file $fileout failed");            
        return $core::errorcodes::FILE_WRITE_FAILED;
    }    
    
    if ($$execqueryvarinit == 0)
    {
        $executequerydetails{$file_in} = &share([]);    
        $$execqueryvarinit = 1;            
    }
    
    $executequerydetails{$file_in}[$$execquerycount] = &share({});    
    share($executequerydetails{$file_in}[$$execquerycount]{varname});
    share($executequerydetails{$file_in}[$$execquerycount]{startlinenumber});
    share($executequerydetails{$file_in}[$$execquerycount]{endlinenumber});
                    
    $executequerydetails{$file_in}[$$execquerycount]{varname} = $varname;
    $executequerydetails{$file_in}[$$execquerycount]{startlinenumber} = $startlinenumber;
    $executequerydetails{$file_in}[$$execquerycount]{endlinenumber} = $endlinenumber;  

    $$execquerycount = $$execquerycount + 1;   
    return $core::errorcodes::MIG_SUCCESS;
}

# Description: Extract sql contents present within execute_query api
# Params:
# Return: NA
sub extractExecuteQueryContents
{
    my $FI = shift;
    my $line = shift;
    my $preparelinenumber = shift;
    my $execquerycount = shift;    
    my $execqueryvarinit = shift;
    my $file_in = shift;
    my $file_execute_query_out = shift;    

    my $varstring;
    my $varname;
    my $sqlcontents;            
    my $startlinenumber = $$preparelinenumber;    
    
    # If the execute_query is present with sql in single line
    if ($$line =~ m/execute_query\(\".*;\"\);/)
    {
        $varstring = $$line;
        $varstring =~ /(.*execute_query\s*\(\")(.*?);\"\);/;
        $varname = $1;
        $sqlcontents = $2;
        
        # Update the shared map
        return updateExecuteQueryMap($varname, $sqlcontents, $execqueryvarinit, $file_in, $execquerycount, $startlinenumber, $$preparelinenumber, $file_execute_query_out);                
    }
    
    $varstring = $$line;
    $varstring =~ /(.*execute_query\s*\(\")(.*)/;
    $varname = $1;
    $sqlcontents = $2;         
    
    # If the execute_query is present with sql in multiple lines
    if ($$line !~ m/\"\)\s*;/)
    {
        while (<FI>)
        {        
            $$preparelinenumber = $$preparelinenumber + 1;                
            $$line = $_;                        
            if ($$line =~ m/\"\)\s*;/)
            {
                $varstring =~ /(.*?\))\"\s*;/;
                $sqlcontents = $sqlcontents.$1;
                last;                    
            }
            
            $sqlcontents = $sqlcontents.$$line;                    
        } 
    }            
    
    # Update the shared map
    return updateExecuteQueryMap($varname, $sqlcontents, $execqueryvarinit, $file_in, $execquerycount, $startlinenumber, $$preparelinenumber, $file_execute_query_out);
}

sub extractsqlcontentsthread
{
     my $filetobeprocessed = "";
     my $ret;
     $filetobeprocessed = getelementfromlist;
          
     core::logger::writeinfologmsg(__FILE__, __LINE__, "Thread created");
     
     while ($filetobeprocessed ne '')
     {
        if ($threadExit != $core::errorcodes::MIG_SUCCESS)
        {
            core::logger::writeerrorlogmsg(__FILE__, __LINE__, "Exiting current thread as some thread has failed with $threadExit");            
            return $threadExit;
        }
        
        if ($filetobeprocessed =~ /\.$/)
        {
            $filetobeprocessed = getelementfromlist;
            next;
        }
        
        $ret = act_file($filetobeprocessed);
        incrementmigratedperlfilescount();
        if ($ret != $core::errorcodes::MIG_SUCCESS)
        {
            core::logger::writeerrorlogmsg(__FILE__, __LINE__, "Extraction of file $filetobeprocessed failed");    
            $extractionfailcount++;
        }
        
        $filetobeprocessed = getelementfromlist;
     }
     
     core::logger::writeinfologmsg(__FILE__, __LINE__, "Exiting thread extractsqlcontentsthread");
     return $core::errorcodes::MIG_SUCCESS;
}

sub mergesqlcontentsthread
{
     my $filetobeprocessed = "";
     my $ret;
     
     core::logger::writeinfologmsg(__FILE__, __LINE__, "Thread created");     
     $filetobeprocessed = getelementfromlist;
     while ($filetobeprocessed ne '')
     {
        if ($threadExit != $core::errorcodes::MIG_SUCCESS)
        {
            core::logger::writeerrorlogmsg(__FILE__, __LINE__, "Exiting current thread as some thread has failed with $threadExit");            
            return $threadExit;
        }
        
        if ($filetobeprocessed =~ /\.$/)
        {
            $filetobeprocessed = getelementfromlist;
            next;
        }
        
        $ret = act_filec($filetobeprocessed);
        incrementmigratedperlfilescount();    
        if ($ret != $core::errorcodes::MIG_SUCCESS)
        {
            core::logger::writeerrorlogmsg(__FILE__, __LINE__, "Merging of file $filetobeprocessed failed");    
            $mergefailcount++;
        }
        
        $filetobeprocessed = getelementfromlist;
     }

    core::logger::writeinfologmsg(__FILE__, __LINE__, "Exiting thread mergesqlcontentsthread");
    return $core::errorcodes::MIG_SUCCESS;
}

# Description: Generate the sql contents from the perl file
# Params: input perl file from which the sql contents must be extracted
# Return: NA or die on failure
sub act_file
{
    my($file_in) = @_;  

    my $file_out = $file_in;
    $file_out =~ s/\.pl$/.pl.sql/;
    
    my $file_sql_var_out = $file_in;
    $file_sql_var_out =~ s/\.pl$/.pl.var.sql/;
    
    my $file_execute_query_out = $file_in;
    $file_execute_query_out =~ s/\.pl$/.pl.exec.sql/;
    
    my $preparelinenumber = 0;
    my $varlinenumber = 0;
    my $varstartlinenumber = 0;
    my $varcount = 0;
    my $filecontentswritten = 0;
    my $sharevarinit = 0;
    
    my $execqueryvarinit = 0;
    my $execquerycount = 0;
    
    my $encoding;
    my $ret = $core::errorcodes::MIG_SUCCESS;
        
    core::logger::writeinfologmsg(__FILE__, __LINE__, "Extracting sql contents for file: $file_in");    
    $perlfilecount++;
    
    redirect_streams();
    $ret = core::utils::getfileencoding($file_in, \$encoding);
    restore_streams();
    if ($ret != $core::errorcodes::MIG_SUCCESS)
    {
        return $ret;
    }
    
    if (!open(FI, "<:encoding($encoding)", "$file_in"))
    {
        core::logger::writeerrorlogmsg(__FILE__, __LINE__, "Opening file $file_in failed with $!");    
        return $core::errorcodes::FILE_OPEN_FAILED;
    }
    
    if (!open(FO, ">:encoding($encoding)", "$file_out"))
    {            
        core::logger::writeerrorlogmsg(__FILE__, __LINE__, "Opening file $file_out failed with $!");
        $ret = $core::errorcodes::FILE_OPEN_FAILED;
        goto CLEAN_ERR_RET0;        
    }
    
    if ($migrateexecutecontents)
    {
        if (!open(FOEXECQUERY, ">:encoding($encoding)", "$file_execute_query_out"))
        {
            core::logger::writeerrorlogmsg(__FILE__, __LINE__, "Opening file $file_execute_query_out failed with $!");            
            $ret = $core::errorcodes::FILE_OPEN_FAILED;
            goto CLEAN_ERR_RET1;
        }

		if ($systemversion !~ /Microsoft/ && $systemversion !~ /MSW/)
		{ 
			if (!chmod(0600, $file_execute_query_out))
			{
				core::logger::writeerrorlogmsg(__FILE__, __LINE__, "changing permissions for file $file_execute_query_out failed with $!");          
				$ret = $core::errorcodes::CHMOD_FAILED;
				goto CLEAN_ERR_RET2;
			}
		}
		else
		{ 
			$chmodcommand = sprintf "icacls %s \/grant:r \*%s:(F,WDAC) \/inheritance:r", $file_execute_query_out, $usersid;
			$chmodcmdresult =`$chmodcommand`;
		}        
    }
    
    if ($migratevariables)
    {
        if (!open(FIEXECQUERY, "<:encoding($encoding)", "$file_in"))
        {
            core::logger::writeerrorlogmsg(__FILE__, __LINE__, "Opening file $file_in failed with $!");        
            $ret = $core::errorcodes::FILE_OPEN_FAILED;
            goto CLEAN_ERR_RET2;
        }
        
        if (!open(FISQL_VAR, "<:encoding($encoding)", "$file_in"))
        {
            core::logger::writeerrorlogmsg(__FILE__, __LINE__, "Opening file $file_in failed with $!");
            $ret = $core::errorcodes::FILE_OPEN_FAILED;            
            close(FIEXECQUERY);                    
            goto CLEAN_ERR_RET2;
        }
        
        if (!open(FOSQL_VAR, ">:encoding($encoding)", "$file_sql_var_out"))
        {
            core::logger::writeerrorlogmsg(__FILE__, __LINE__, "Opening file $file_sql_var_out failed with $!");
            $ret = $core::errorcodes::FILE_OPEN_FAILED;
            close(FISQL_VAR);
            close(FIEXECQUERY);                    
            goto CLEAN_ERR_RET2;
        }
        
		if ($systemversion !~ /Microsoft/ && $systemversion !~ /MSW/)
		{ 
			if (!chmod(0600, $file_sql_var_out))
			{
				core::logger::writeerrorlogmsg(__FILE__, __LINE__, "Changing permissions for file $file_sql_var_out failed with $!");               
				$ret = $core::errorcodes::CHMOD_FAILED;
				goto CLEAN_ERR_RET3;
			}
		}
		else
		{ 
			$chmodcommand = sprintf "icacls %s \/grant:r \*%s:(F,WDAC) \/inheritance:r", $file_sql_var_out, $usersid;
			$chmodcmdresult =`$chmodcommand`;
		}
        
    }
	
	if ($systemversion !~ /Microsoft/ && $systemversion !~ /MSW/)
	{
		if (!chmod(0600, $file_out))
		{
			core::logger::writeerrorlogmsg(__FILE__, __LINE__, "changing permissions for file $file_out failed with $!");            
			$ret = $core::errorcodes::CHMOD_FAILED;
			goto CLEAN_ERR_RET3;
		}
	}
	else
	{ 
		$chmodcommand = sprintf "icacls %s \/grant:r \*%s:(F,WDAC) \/inheritance:r", $file_out, $usersid;
		$chmodcmdresult =`$chmodcommand`;
	}
    
    while (<FI>)
    {
        my $line = $_;
        
        if ($threadExit != $core::errorcodes::MIG_SUCCESS)
        {
            core::logger::writeerrorlogmsg(__FILE__, __LINE__, "Exiting current thread as some thread has failed with $threadExit");
            $ret = $threadExit;
            goto CLEAN_ERR_RET3;
        }
        
        $preparelinenumber = $preparelinenumber + 1;
        # handle parsing of sql variables
        if (($line =~ m/prepare\s*\(/  or $line =~ m/execute_query\s*\(\$/) and $line !~ m/^\s*#/)
        {
            my $varstring = $line;
            my $varname;
            my $startlinenumber;
            my $endlinenumber;
            my $actualvariablename;
            
            if ($line =~ m/prepare\s*\(/)
            {
                $varstring =~ /.*prepare\s*(\((.*)\))/;
                $varname = $2;
                $ret = extractVariableContents(\*FISQL_VAR, $file_in, \*FOSQL_VAR, $file_sql_var_out, \$varlinenumber, \$varname, \$varstartlinenumber, \$line, \$varcount, \$startlinenumber, \$endlinenumber, \$actualvariablename);                         
            }
            else
            {
                $varstring =~ m/execute_query\s*\((\$.*?)\)\;/;
                $varname = $1;        
                $ret = extractVariableContents(\*FIEXECQUERY, $file_in, \*FOSQL_VAR, $file_sql_var_out, \$varlinenumber, \$varname, \$varstartlinenumber, \$line, \$varcount, \$startlinenumber, \$endlinenumber, \$actualvariablename);                                         
            }
            
            if ($ret != $core::errorcodes::MIG_SUCCESS)
            {
                $threadExit = $ret;
                goto CLEAN_ERR_RET3;
            }
                       
            if ($varcount > 0)
            {
                if ($sharevarinit == 0)
                {                
                    $varnamedetails{$file_in} = &share([]);    
                    $sharevarinit = 1;
                }
                    
                $varnamedetails{$file_in}[$varcount-1] = &share({});    
                share($varnamedetails{$file_in}[$varcount-1]{varname});
                share($varnamedetails{$file_in}[$varcount-1]{startlinenumber});
                share($varnamedetails{$file_in}[$varcount-1]{endlinenumber});
                share($varnamedetails{$file_in}[$varcount-1]{actualvariablename});
                
                $varnamedetails{$file_in}[$varcount-1]{varname} = $varname;
                $varnamedetails{$file_in}[$varcount-1]{startlinenumber} = $startlinenumber;
                $varnamedetails{$file_in}[$varcount-1]{endlinenumber} = $endlinenumber;        
                $varnamedetails{$file_in}[$varcount-1]{actualvariablename} = $actualvariablename;                        
            }   

            next;
        }
        
        # handle parsing of execute_query
        if ($migrateexecutecontents and $line =~ m/execute_query\(/ and $line !~ m/^\s*#/)
        {        
            $ret = extractExecuteQueryContents(\*FI, \$line, \$preparelinenumber, \$execquerycount, \$execqueryvarinit, $file_in, $file_execute_query_out);
            if ($ret != $core::errorcodes::MIG_SUCCESS)
            {    
                $threadExit = $ret;
                goto CLEAN_ERR_RET3;
            }
        }
        
        # Parsing of SQL_LANG and generate sql file for sql_lang
        if ($tdsqltagname ne '')
        {
            $ret = extractSqlFromSQLLANGTag(\*FI, \$line, \$preparelinenumber, \$filecontentswritten, $file_in, $file_out);            
            if ($ret != $core::errorcodes::MIG_SUCCESS)
            {    
                $threadExit = $ret;
                goto CLEAN_ERR_RET3;
            }            
        }

        # Parsing of BTEQ and generate sql file for bteq
        if ($bteqtagname ne '')
        {
            $ret = extractSqlFromBTEQTag(\*FI, \$line, \$preparelinenumber, \$filecontentswritten, $file_in, $file_out);
            if ($ret != $core::errorcodes::MIG_SUCCESS)
            {    
                $threadExit = $ret;
                goto CLEAN_ERR_RET3;
            }            
        }
    }
    
    if ($migratevariables)
    {    
        $varcountdetails{$file_in} = &share({});          
        $varcountdetails{$file_in}{count} = $varcount; 
    }
    
    if ($migrateexecutecontents)
    {
        $executequerycountdetails{$file_in} = &share({});
        $executequerycountdetails{$file_in}{execquerycount} = $execquerycount; 
    }   
    
    $sqlfilecreateddetails{$file_in} = &share({});    
    $sqlfilecreateddetails{$file_in}{count} = $filecontentswritten;    
    
CLEAN_ERR_RET3:    
    if ($migratevariables)
    {    
        close(FISQL_VAR);
        close(FOSQL_VAR);        
        close(FIEXECQUERY);            
    }

CLEAN_ERR_RET2:    
    if ($migrateexecutecontents)
    {            
        close(FOEXECQUERY);
    }    
        
CLEAN_ERR_RET1:    
    close (FO);
    
CLEAN_ERR_RET0:    
    close (FI);
    
    return $ret;
}

# Description: Merge back the migrated execute_query contents back to perl file
# Params:
# Return: NA
sub mergeExecuteQuerySQLToOriginalFile
{
    my $currexecqueryvarcount = shift;
    my $execqueryvarcount = shift;
    my $file_in = shift;
    my $currinputfilelinecount = shift;
    my $execqueryendlinenumber = shift;
    my $execqueryvarname = shift;
    my $execquerystartlinenumber = shift;
    my $FI = shift;
    my $FEI = shift;
    my $file_execquery_out = shift;
    
    my $nxtexecquery = "";   
    my $execqueryline = "";
    my $convertedexecsql;
    my $line;
    my $sharevarinit = 0;    
    
    # get next variable
    $$currexecqueryvarcount = $$currexecqueryvarcount + 1;
    if ($$currexecqueryvarcount < $execqueryvarcount)
    {
        $nxtexecquery = $executequerydetails{$file_in}[$$currexecqueryvarcount]{varname};
    }
    
    # skip reading the input variable, migrated sql variable will be copied
    if ($$currinputfilelinecount != $$execqueryendlinenumber)
    {
        while (<FI>)
        {
            $$currinputfilelinecount = $$currinputfilelinecount + 1;
            if ($$currinputfilelinecount == $$execqueryendlinenumber)
            {
                $line = $_; 
                if (!defined $line)
                {
                    core::logger::writeerrorconsolelogret(__FILE__, __LINE__, "Reading from file $file_in failed");
                    return $core::errorcodes::FILE_READ_FAILED;
                }                
                
                last;
            }
        }
    }        
    
    # read from migrated file and write to original file
    while (<FEI>)
    {
        my $prevseekpos = 0;
        my $currexecqueryvarname = $$execqueryvarname;
        $execqueryline = $_;
        if (!defined $execqueryline)
        {
            core::logger::writeerrorconsolelogret(__FILE__, __LINE__, "Reading from file $file_execquery_out failed");
            return $core::errorcodes::FILE_READ_FAILED;
        }    
                
        if ($execqueryline =~ m/\Q$currexecqueryvarname/)
        {
            while (<FEI>)
            {
                $execqueryline = $_;
                if (!defined $execqueryline)
                {
                    core::logger::writeerrorconsolelogret(__FILE__, __LINE__, "Reading from file $file_execquery_out failed");
                    return $core::errorcodes::FILE_READ_FAILED;
                }    
                
                if ($execqueryline !~ m/\/\*\s*\Q$nxtexecquery/)
                {
                    if ($execqueryline eq ";\n")
                    {
                        $convertedexecsql = $convertedexecsql.";";
                    }
                    elsif ($execqueryline ne "\n")
                    {
                        $convertedexecsql = $convertedexecsql.$execqueryline;
                    }
                    
                    $prevseekpos = tell(FEI);
                }
                else
                {
                    if (!seek(FEI, $prevseekpos, 0))
                    {
                        core::logger::writeerrorconsolelogret(__FILE__, __LINE__, "seek of file $file_execquery_out failed");
                        return $core::errorcodes::FILE_SEEK_FAILED;
                    }
                    
                    last;
                }                
            }
                            
            print FCO "$$execqueryvarname$convertedexecsql\"\);";                   
            if ($$currexecqueryvarcount < $execqueryvarcount) 
            {                
                $$execqueryvarname = $executequerydetails{$file_in}[$$currexecqueryvarcount]{varname};
                $$execquerystartlinenumber = $executequerydetails{$file_in}[$$currexecqueryvarcount]{startlinenumber};
                $$execqueryendlinenumber = $executequerydetails{$file_in}[$$currexecqueryvarcount]{endlinenumber};        
            }
            
            $convertedexecsql = "";         
            $nxtexecquery = "";
            last;
        }
    }
    
    return $core::errorcodes::MIG_SUCCESS;
}

# Description: Merge back the migrated variable contents back to perl file
# Params:
# Return: NA
sub mergeMigratedVariablesToOriginalFile
{
    my $FI = shift;
    my $line = shift;
    my $currinputfilelinecount = shift;
    my $startlinenumber = shift;
    my $FVI = shift;
    my $curvarcount = shift;
    my $varname = shift;
    my $endlinenumber = shift;
    my $file_in = shift;
    my $varcount = shift;
    my $migrationblockcompleted = shift;
    my $actualvariablename = shift;
    my $file_var_out = shift;
    
    my $varline;
    my $nxtvarname = "";
    my $convertedvarsql;        
    
    my $sharevarinit = 0;
    
    if ($$currinputfilelinecount == $$startlinenumber)
    {
        # get next variable
        $$curvarcount = $$curvarcount + 1;
        if ($$curvarcount < $$varcount)
        {
            $nxtvarname = $varnamedetails{$$file_in}[$$curvarcount]{varname};
        }
        
        # skip reading the input variable, migrated sql variable will be copied
        if ($$currinputfilelinecount != $$endlinenumber)
        {
            while (<FI>)
            {
                $$currinputfilelinecount = $$currinputfilelinecount + 1;
                if ($$currinputfilelinecount == $$endlinenumber)
                {
                    $$line = $_; 
                    if (!defined $$line)
                    {
                        core::logger::writeerrorconsolelogret(__FILE__, __LINE__, "Reading from file $file_in failed");
                        return $core::errorcodes::FILE_READ_FAILED;
                    }
                    
                    last;
                }
            }
        }
        
        # read from migrated file and write to original file
        while (<FVI>)
        {
            my $skipvariablenamecontents = 0;
            my $prevseekpos = 0;
            $varline = $_;
            if (!defined $varline)
            {
                core::logger::writeerrorconsolelogret(__FILE__, __LINE__, "Reading from file $file_var_out failed");
                return $core::errorcodes::FILE_READ_FAILED;
            }
                    
            if ($varline =~ m/\Q$$varname/)
            {
                # variable name will be converted to $variablename \n ; \n, 
                # these need to be skipped, one line is already read, other 2 need to be skipped
                $skipvariablenamecontents = 2;
                while (<FVI>)
                {
                    $varline = $_;
                    if (!defined $varline)
                    {
                        core::logger::writeerrorconsolelogret(__FILE__, __LINE__, "Reading from file $file_var_out failed");
                        return $core::errorcodes::FILE_READ_FAILED;
                    }
                    
                    if ($skipvariablenamecontents > 0)
                    {
                        $skipvariablenamecontents = $skipvariablenamecontents - 1;
                        next;
                    }
                    elsif ($varline !~ m/\Q$nxtvarname/)
                    {
                        $convertedvarsql = $convertedvarsql.$varline;
                        $prevseekpos = tell(FVI);
                    }
                    else
                    {
                        if (!seek(FVI, $prevseekpos, 0))
                        {
                            core::logger::writeerrorconsolelogret(__FILE__, __LINE__, "seek of file $file_var_out failed");
                            return $core::errorcodes::FILE_SEEK_FAILED;
                        }
                        
                        last;
                    }
                }
                
                print FCO "$$actualvariablename=\"$convertedvarsql\";\n";                
                
                if ($sharevarinit == 0)
                {                
                    $varnamedetails{$file_in} = &share([]);    
                    $sharevarinit = 1;
                }                        
                
                $varnamedetails{$$file_in}[$$varcount] = &share({});                                            
                $$varname = $varnamedetails{$$file_in}[$$curvarcount]{varname};
                $$actualvariablename = $varnamedetails{$$file_in}[$$curvarcount]{actualvariablename};
                $$startlinenumber = $varnamedetails{$$file_in}[$$curvarcount]{startlinenumber};
                $$endlinenumber = $varnamedetails{$$file_in}[$$curvarcount]{endlinenumber};        
                $convertedvarsql = "";         
                $$migrationblockcompleted = 1;
                last;
            }
        }                
    }
    
    return $core::errorcodes::MIG_SUCCESS;
}

# Description: Merge back the migrated bteq tag contents and sql_lang tag contents back to perl file
# Params:
# Return: NA
sub mergeMigratedBTEQSQLLANGToOriginalFile
{
    my $FSI = shift;
    my $FCO = shift;
    my $blockcompleted = shift;
    my $file_sin = shift;
    my $blocktype = '';
    
    my $blockcontentscompleted = 0;
    while (<FSI>)
    {
        my $sline = $_;
        if (!defined $sline)
        {
            core::logger::writeerrorconsolelogret(__FILE__, __LINE__, "Reading from file $file_sin failed");
            return $core::errorcodes::FILE_READ_FAILED;
        }
        
        if ($bteqtagname ne '')
        {
			if ($sline =~ m/CLOSE\s*\(\s*$bteqtaglist\s*\)/i)
			{
				$sline =~ s/CLOSE/close/mgi;
			}
			if ($sline =~ m/print\s+$bteqtaglist/i)
            {
				$sline =~ s/PRINT/print/mgi;
				$blocktype = 'bteq';
				print FCO "$sline\n";
             	if ($addtimingon)
            	{
               	    print FCO "\n\\\\timing\n";
            	}
		
				# next line will be ; it can be skipped
				$sline = <FSI>; 
				if (!defined $sline)
				{
					core::logger::writeerrorconsolelogret(__FILE__, __LINE__, "Reading from file $file_sin failed");
					return $core::errorcodes::FILE_READ_FAILED;
				}
			}
            # next;
        }
        
        if (($tdsqltagname ne '') and ($sline =~ m/\s*\w*\s*\$\w+\s*=\s*<\s*<\s*$tdsqltaglist\s*;?/i))
        {
            # Migration tool adds spaces before = & <<, need to be converted before putting it to original file
            # replace ' = << ' to  =<<
            $sline =~ s/ = << /=<</mgi;    
            # $sline =~ s/\n/;/mgi;    
	        $blocktype = 'sql_lang';
            
            print FCO $sline;
            
            # next line will be ; it can be skipped
            $sline = <FSI>; 
            if (!defined $sline)
            {
                core::logger::writeerrorconsolelogret(__FILE__, __LINE__, "Reading from file $file_sin failed");
                return $core::errorcodes::FILE_READ_FAILED;
            }            
            
            # next;
        }               

        if ((($bteqtagname ne '') and ($sline =~ m/CLOSE\s*\(\s*$bteqtaglist\s*\)/i) and ($blocktype eq 'bteq')) or
            (($tdsqltagname ne '') and ($sline =~ m/(?<!<)$tdsqltaglist$/i) and ($blocktype eq 'sql_lang')))                
        {             
	    	if($blocktype eq 'sql_lang') 
	     	{
	    		my $temp = $1;
	    		$sline =~ s/$temp/\n$temp/mgi;           
	     	}
            $blockcontentscompleted = 1;
	     	$blocktype = '';
        } 

        if ($blockcontentscompleted =~ 1)
        {
            print FCO $sline;
            last;
        }

        if ($bteqtagname ne '')
        {   
            # Check if line is ending with ENDOFINPUT
            if ($sline =~ m/^ENDOFINPUT\s+CLOSE\s*\(\s*$bteqtaglist\s*\)\s*;?$/i)
            {
                $sline =~ s/ENDOFINPUT CLOSE/ENDOFINPUT\nclose/mgi;                    
            }
            else
            {
                $sline =~ s/ENDOFINPUT CLOSE/\nENDOFINPUT\nclose/mgi;                                        
            }
        }   

        print FCO $sline;
    }
    
    $$blockcompleted = 1;
    return $core::errorcodes::MIG_SUCCESS;
}

# Description: Merge the migrated sql contents to the original file
# Params: input perl file for which the sql contents must be merged 
#         in the converted folder
# Return: NA or die on failure
sub act_filec
{
    my($file_in) = @_;
    my $file_var_out;
    my $varcount;
    my $currinputfilelinecount = 0;
    my $curvarcount = 0;    
    my $varname;
    my $actualvariablename;
    my $startlinenumber = 0;
    my $endlinenumber;
    my $migrationblockcompleted = 0;
    
    my $file_execquery_out;
    my $execqueryvarcount;
    my $currexecqueryvarcount = 0;
    my $execqueryvarname;
    my $execquerystartlinenumber;
    my $execqueryendlinenumber;
    my $shareexecqueryvarinit = 0;
    my $statfile;
    
    my $encoding;
    if (($file_in !~ m/.*\.pl$/) or ($file_in =~ m/\Q$location\/converted/))
    {
        return $core::errorcodes::MIG_SUCCESS;
    }    
    
    my $file_sin = $file_in;    
    my $basepath;    
    if (-f $location)
    {
        $basepath = dirname($location);
    }
    else
    {
        $basepath = $location;
    }
        
    my $outputstring = "output";
    $file_sin =~ s/\Q$basepath/$outputfolder$directoryseparator$outputstring/;
    $file_sin =~ s/\.pl$/.pl.sql/;    
    
    my $file_cout = $file_sin;
    $file_cout =~ s/\.pl.sql$/.pl/;
    
    my $baseoutputpath = dirname($file_cout);
    if (!($statfile = stat($baseoutputpath)))
    {
        my $diemsg = sprintf("Stat of $baseoutputpath failed with $!.");
        core::logger::writeerrorconsolelogret(__FILE__, __LINE__, $diemsg);        
        return $core::errorcodes::INVALID_PERMISSIONS;
    }    
    
    my $val = $statfile->mode & 0200;
    if ($val == 0)                
    {
        my $diemsg = sprintf("File $file_cout cannot be created as $baseoutputpath does not have write permissions.");
        core::logger::writeerrorconsolelogret(__FILE__, __LINE__, $diemsg);        
        return $core::errorcodes::INVALID_PERMISSIONS;
    } 
    
    my $ret = core::utils::getfileencoding($file_in, \$encoding);
    if ($ret != $core::errorcodes::MIG_SUCCESS)
    {
        return $ret;
    }    
        
    core::logger::writeinfologmsg(__FILE__, __LINE__, "Merging sql contents for file:$file_in");            
    if ($migratevariables)
    {
        $file_var_out = $file_in;        
        $varcount = $varcountdetails{$file_in}{count};
            
        $file_var_out =~ s/\Q$basepath/$outputfolder$directoryseparator$outputstring/;
        $file_var_out =~ s/\.pl$/.pl.var.sql/;
        
        if ($varcount > 0)
        {
            $varname = $varnamedetails{$file_in}[$curvarcount]{varname};
            $actualvariablename = $varnamedetails{$file_in}[$curvarcount]{actualvariablename};
            $startlinenumber = $varnamedetails{$file_in}[$curvarcount]{startlinenumber};
            $endlinenumber = $varnamedetails{$file_in}[$curvarcount]{endlinenumber};
            if (!open(FVI, "<:encoding($encoding)", "$file_var_out"))
            {
                core::logger::writeerrorconsolelogret(__FILE__, __LINE__, "Opening file $file_var_out with encoding $encoding failed with $!");
                return $core::errorcodes::FILE_OPEN_FAILED;
            }
            
            binmode(FVI, ":raw");
        }    
    }    
    
    if ($migrateexecutecontents)
    {
        $file_execquery_out = $file_in;      
        $execqueryvarcount = $executequerycountdetails{$file_in}{execquerycount};
        
        $file_execquery_out =~ s/\Q$basepath/$outputfolder$directoryseparator$outputstring/;
        $file_execquery_out =~ s/\.pl$/.pl.exec.sql/;
        
        if ($execqueryvarcount > 0)
        {
            $execqueryvarname = $executequerydetails{$file_in}[$currexecqueryvarcount]{varname};
            $execquerystartlinenumber = $executequerydetails{$file_in}[$currexecqueryvarcount]{startlinenumber};
            $execqueryendlinenumber = $executequerydetails{$file_in}[$currexecqueryvarcount]{endlinenumber};
            if (!open(FEI, "<:encoding($encoding)", "$file_execquery_out"))
            {    
                core::logger::writeerrorconsolelogret(__FILE__, __LINE__, "Opening file $file_execquery_out failed with $!");
                $ret = $core::errorcodes::FILE_OPEN_FAILED;
                goto ERR_RET0;
            }
            
            binmode(FEI, ":raw");
        }        
    }
    
    if (!open(FI, "<:encoding($encoding)", "$file_in"))
    {
        core::logger::writeerrorconsolelogret(__FILE__, __LINE__, "Opening file $file_in failed with $!");
        $ret = $core::errorcodes::FILE_OPEN_FAILED;
        goto ERR_RET1;
    }
    
    my $filecontentswritten = $sqlfilecreateddetails{$file_in}{count};    
    if ($filecontentswritten == 1)
    {
        if (!open(FSI, "<:encoding($encoding)", "$file_sin"))
        {    
            core::logger::writeerrorconsolelogret(__FILE__, __LINE__, "Open file $file_sin failed with $!");
            $ret = $core::errorcodes::FILE_OPEN_FAILED;
            goto ERR_RET2;
        }
    }
    
    core::logger::writeinfologmsg(__FILE__, __LINE__, "File $file_cout will be written with charset: $encoding");
    if (!open(FCO, ">:encoding($encoding)", "$file_cout"))
    {
        core::logger::writeerrorconsolelogret(__FILE__, __LINE__, "Opening file $file_cout failed with $!");    
        $ret = $core::errorcodes::FILE_OPEN_FAILED;
        goto ERR_RET3;
    }
	
	if ($systemversion !~ /Microsoft/ && $systemversion !~ /MSW/)
	{ 
		if (!chmod(0600, $file_cout))
		{
			core::logger::writeerrorconsolelogret(__FILE__, __LINE__, "changing permissions for file $file_cout failed with $!"); 
			$ret = $core::errorcodes::CHMOD_FAILED;
			goto ERR_RET4;
		}
	}
	else
	{ 
		$chmodcommand = sprintf "icacls %s \/grant:r \*%s:(F,WDAC) \/inheritance:r", $file_cout, $usersid;
		$chmodcmdresult =`$chmodcommand`;
	}
       
    my $blockcompleted = 0;
    $currinputfilelinecount = 0;
    while (<FI>)
    {           
        my $line = $_;      
        if (!defined $line)
        {
            core::logger::writeerrorconsolelogret(__FILE__, __LINE__, "Reading from file $file_in failed");            
            $ret = $core::errorcodes::FILE_READ_FAILED;
            goto ERR_RET4;            
        }

        if ($threadExit != $core::errorcodes::MIG_SUCCESS)
        {
            core::logger::writeerrorlogmsg(__FILE__, __LINE__, "Exiting current thread as some thread has failed with $threadExit");
            $ret = $threadExit;
            goto ERR_RET4;            
        }
        
        $currinputfilelinecount = $currinputfilelinecount + 1;
        if (($bteqtagname ne '') and ($line =~ m/close\s*\(\s*$bteqtaglist\s*\)/i))        
        {
            $blockcompleted = 0;
            next;
        }
        
        if (($tdsqltagname ne '') and ($line =~ m/^$tdsqltaglist/i))
        {
            $blockcompleted = 0;
            next;
        }                    
        
        if ($migrateexecutecontents and ($execqueryvarcount > 0) and ($currexecqueryvarcount < $execqueryvarcount) and ($currinputfilelinecount == $execquerystartlinenumber))
        {            
            $ret = mergeExecuteQuerySQLToOriginalFile(\$currexecqueryvarcount, $execqueryvarcount, $file_in, \$currinputfilelinecount, \$execqueryendlinenumber, \$execqueryvarname, \$execquerystartlinenumber, \*FI, \*FEI, $file_execquery_out);
            if ($ret != $core::errorcodes::MIG_SUCCESS)
            {
                goto ERR_RET4;            
            }
            
            next;
        }

        if ($migratevariables and ($varcount > 0) and ($curvarcount < $varcount))
        {
            $ret = mergeMigratedVariablesToOriginalFile(\*FI, \$line, \$currinputfilelinecount, \$startlinenumber, \*FVI, \$curvarcount, \$varname, \$endlinenumber, \$file_in, \$varcount, \$migrationblockcompleted, \$actualvariablename, $file_var_out);
            if ($ret != $core::errorcodes::MIG_SUCCESS)
            {
                goto ERR_RET4;
            }
            
            if ($migrationblockcompleted == 1)
            {
                $migrationblockcompleted = 0;
                next;
            }
        }        
                
        if ((($bteqtagname ne '') and ($line =~ m/$bteqtaglist\s*<<\s*ENDOFINPUT/i)) or
            (($tdsqltagname ne '') and ($line =~ m/\s*\w*\s*\$\w+\s*=\s*<\s*<\s*$tdsqltaglist\s*;?/i)))
        {
            $ret = mergeMigratedBTEQSQLLANGToOriginalFile(\*FSI, \*FCO, \$blockcompleted, $file_sin);            
            if ($ret != $core::errorcodes::MIG_SUCCESS)
            {
                goto ERR_RET4;
            }            
        }
        else
        {
            if ($blockcompleted == 1)
            {
                next;
            }
            
            if ($bteqtagname ne '')
            {
                $line =~ s/\| $bteqtaglist/| gsql -d test -p 8000 -a -r /mg;
            }

            # If timing on is specified, add logic to get execution time
            if (($addtimingon) and ($line =~ m/->execute\(/))
            {
                print FCO "use Time::HiRes qw/gettimeofday/;\n";
                print FCO "my \$start = [Time::HiRes::gettimeofday()];\n";
            }            
            
            print FCO $line;
            
            # If timing on is specified, add logic to get execution time
            if (($addtimingon) and ($line =~ m/->execute\(/))
            {
                print FCO "my \$elapsed = Time::HiRes::tv_interval(\$start);\n";            
                print FCO "\$elapsed = \$elapsed * 1000;\n";
                print FCO 'printf("Time: %.3f ms\n", $elapsed);';        
                print FCO "\n";
            }     
        }
    }       

ERR_RET4:
    close(FCO);    
    
ERR_RET3:        
    if ($filecontentswritten == 1)
    {
        close(FSI);
    }    

ERR_RET2:    
    close(FI);   
    
ERR_RET1:
    if ($migrateexecutecontents and ($execqueryvarcount > 0))
    {
        close(FEI);
    }
    
ERR_RET0:    
    if ($migratevariables and ($varcount > 0))
    {
        close(FVI);
    }

    if ($removeintermediatefiles)
    {
        # remove migrated.pl.sql file
        unlink $file_sin; 
        
        # remove generated input sql file
        my $inputsqlfile = $file_in;    
        $inputsqlfile =~ s/\.pl$/.pl.sql/;    
        unlink $inputsqlfile;
        
        if ($migratevariables)
        {
            # remove converted .pl.var.sql file
            unlink $file_var_out; 
            
            # remove generated .pl.var.sql file
            my $inputvarsqlfile = $file_in;
            $inputvarsqlfile =~ s/\.pl$/.pl.var.sql/;
            unlink $inputvarsqlfile;        
        }        
        
        if ($migrateexecutecontents)
        {
            # remove converted .pl.eec.sql file
            unlink $file_execquery_out; 
            
            # remove generated .pl.exec.sql file
            my $inputexecsqlfile = $file_in;
            $inputexecsqlfile =~ s/\.pl$/.pl.exec.sql/;
            unlink $inputexecsqlfile;        
        }      
    }
    
    return $ret;
}

# Description: Perform the precheck before the actual migration
# Params: NA
# Return: NA or die on failure
sub precheck
{
    #my %appconfig;
    my %perlconfig;
    my $ret;
    
    if ($help) 
    {
        core::help::printhelp();
    } 

    if ($version)
    {
        core::version::printversion();;
    }
    
    $systemversion = $^O;
    getWinSid();
    
    if ($systemversion !~ /Microsoft/ && $systemversion !~ /MSW/)
    {
        $directoryseparator = '/';
        if ( $< == 0 ) 
        {
            core::logger::writeerrorconslogret(__FILE__, __LINE__, "Root privileged users are not allowed to execute the DSC.");
            return $core::errorcodes::INVALID_PERMISSIONS;
        }
    }
    else
    {
        $directoryseparator = '\\';
    }
        
    if ($location eq '')
    {
        core::logger::writeerrorconslogret(__FILE__, __LINE__, "value for --input-folder option not specified.");
        return $core::errorcodes::INVALID_ARGUMENT;        
    }
        
    $ret = core::config::readconfigfile("$migrationtoolpath".$directoryseparator."config".$directoryseparator."perl-migration.properties", \%perlconfig);
    if ($ret != $core::errorcodes::MIG_SUCCESS)
    {
        return $ret;
    }
    
    core::config::updateperlconfiguration(%perlconfig);

    $ret = core::config::readconfigfile("$migrationtoolpath".$directoryseparator."config".$directoryseparator."application.properties", \%appconfig);
    if ($ret != $core::errorcodes::MIG_SUCCESS)
    {
        return $ret;
    }
    
    core::config::updateapplicationconfiguration(%appconfig);
    
    if ($core::logger::loggingpath eq '')
    {
        $core::logger::loggingpath = $migrationtoolpath.$directoryseparator."log";
    }
    
    if ($core::logger::logginglevel eq '')
    {
        $core::logger::logginglevel = 'INFO';
    }
    
    $core::logger::logginglevel = uc($core::logger::logginglevel);
    if ($core::logger::logginglevel ne 'INFO' and $core::logger::logginglevel ne 'WARNING' and $core::logger::logginglevel ne 'ERROR')
    {
        core::logger::writeerrorconslogret(__FILE__, __LINE__, "Invalid value $core::logger::logginglevel specified for --logging-level.");
        return $core::errorcodes::INVALID_ARGUMENT;                
    }        
    
    $ret = core::logger::initlog();
    if ($core::errorcodes::MIG_SUCCESS != $ret)
    {
        return $ret;
    }
    
    if (!-e $location)
    {       
        core::logger::writeerrorconslogret(__FILE__, __LINE__, "value specified for --input-folder:$location does not exist.");
        return $core::errorcodes::INVALID_ARGUMENT;                        
    }

    if (-d $location)
    {
        $ret = core::utils::checkdir($location);
        if ($ret != $core::errorcodes::MIG_SUCCESS)
        {
            return $ret;
        }        
    }

    if (-f $location)
    {
        $ret = core::utils::checkfile($location);
        if ($ret != $core::errorcodes::MIG_SUCCESS)
        {
            return $ret;
        }        
    }
    
    if ($outputfolder eq '')
    {
        if (-f $location)
        {
            my $basefilepath = dirname($location);
            $outputfolder = join "", $basefilepath, $directoryseparator."..".$directoryseparator."converted".$directoryseparator;
        }
        else
        {
            $outputfolder = join "", $location, $directoryseparator."converted";    
        }    
    }
    
    if (-e $outputfolder and !-d $outputfolder) 
    {
        core::logger::writeerrorconslogret(__FILE__, __LINE__, "$outputfolder should be a directory.");
        return $core::errorcodes::INVALID_ARGUMENT;            
    }
    
    if (!-e $outputfolder)
    {    
        eval
        {
            if (!mkpath($outputfolder, 0, 0700))
            {
                core::logger::writeerrorconslogret(__FILE__, __LINE__, "Directory creation of $outputfolder failed");
                return $core::errorcodes::DIR_CREATE_FAILED;
            }

			if (!($systemversion !~ /Microsoft/ && $systemversion !~ /MSW/))
			{ 
				$chmodcommand = sprintf "icacls %s \/grant:r \*%s:(F,MA) \/inheritance:r", $outputfolder, $usersid;
				$chmodcmdresult =`$chmodcommand`;
			}
        };
        if ($@) 
        {
            core::logger::writeerrorconslogret(__FILE__, __LINE__, "Creating directory $outputfolder failed");
            return $core::errorcodes::DIR_CREATE_FAILED;
        }        
    }
    else
    {        
        $ret = core::utils::checkdir($outputfolder);
        if ($ret != $core::errorcodes::MIG_SUCCESS)
        {
            return $ret;
        }
        
            my $statfile = stat($outputfolder);
            my $val = $statfile->mode & 0200;
            if ($val == 0)                
            {
                my $diemsg = sprintf("Ouput directory $outputfolder does not have write permissions.");
                core::logger::writeerrorconslogret(__FILE__, __LINE__, $diemsg);        
                return $core::errorcodes::INVALID_PERMISSIONS;
            }                    
    }
    
    if ($bteqtagname eq '')
    {
    	$bteqtagname = 'bteq';
    }
    
    if ($targetfiles eq '')
    {
        $targetfiles = 'overwrite';
    }
    
    $targetfiles = lc($targetfiles);
    if ($targetfiles ne 'overwrite' and $targetfiles ne 'cancel' and $targetfiles ne 'delete')
    {
        core::logger::writeerrorconslogret(__FILE__, __LINE__, "Invalid value $targetfiles specified for --target_files");
        return $core::errorcodes::INVALID_ARGUMENT;
    }    
    
    if ($targetfiles eq 'cancel')
    {
        $ret = checkcontentexist($outputfolder);
        if ($ret != $core::errorcodes::MIG_SUCCESS)
        {
            return $ret
        }
    }    
    elsif ($targetfiles eq 'delete')
    {
        if (!rmtree("$outputfolder".$directoryseparator))
        {
            core::logger::writeerrorconslogret(__FILE__, __LINE__, "Removing directory $outputfolder failed");                
            return $core::errorcodes::DIR_REMOVE_FAILED;
        }
        
        if (!mkpath($outputfolder, 0, 0700))
        {
            core::logger::writeerrorconslogret(__FILE__, __LINE__, "Creating directory $outputfolder failed");        
            return $core::errorcodes::DIR_CREATE_FAILED;
        }

		if (!($systemversion !~ /Microsoft/ && $systemversion !~ /MSW/))
		{ 
			$chmodcommand = sprintf "icacls %s \/grant:r \*%s:(F,MA)   \/inheritance:r", $outputfolder, $usersid;
			$chmodcmdresult =`$chmodcommand`;
		}
    }
    
    my $cpu_count;
    if ($systemversion =~ /Microsoft/ || $systemversion =~ /MSW/)
    {
        my $cpustr;
        $cpustr = `wmic cpu get NumberOfLogicalProcessors/Format:List`;
        $cpustr =~ /\s*\w*=(\d*)/;
        $cpu_count = $1;
    }
    else
    {
        chomp($cpu_count = `grep -c -P '^processor\\s+:' /proc/cpuinfo`);
    }
    
    my $def_thread_count=$cpu_count-1;
    if( $num_of_threads < 1 or $num_of_threads > $cpu_count)
    {
        core::logger::writewarninglogmsg(__FILE__, __LINE__, "value for thread count is more than cpu count or <1, setting to $def_thread_count");    
        $num_of_threads = $def_thread_count;
    }    
    
    return $core::errorcodes::MIG_SUCCESS;
}

# Create lock file for concurrency
# Params: NA
# Return: NA or die on failure
sub createlockfile
{        
    $lockfilepath = join "", $outputfolder, $directoryseparator,  "sqlTDtoGS.lock";
    if (-e $lockfilepath)
    {
        open(FI, "<$lockfilepath") or core::logger::writeerrorconsoleandlog(__FILE__, __LINE__, "Opening file $lockfilepath failed with $!");
        my $line = <FI>;
        
        # process id should be numeric
        if ($line !~ m/[0-9]*/)
        {
            close(FI);    
            core::logger::writeerrorconsoleandlog(__FILE__, __LINE__, "Invalid contents in $lockfilepath, delete the file and re-execute");
        }

        if ($systemversion =~ /Microsoft/ || $systemversion =~ /MSW/)
        {
            my $processcount = 0;
            my $perlprocessstring = `WMIC process where "Commandline like '%sqlTDtoGS.pl%'"  get Processid, caption, commandline`;
            my @lines = split /\n/, $perlprocessstring;
            foreach my $line (@lines) 
            {
                if ($line =~ /sqlTDtoGS/ and $line !~ /WMIC/)
                {
                    $processcount = $processcount + 1;
                }
            }
            
            if ($processcount > 1)
            {
                close(FI);    
                core::logger::writeerrorconsoleandlog(__FILE__, __LINE__, "Another DSC process is running on same folder, re-execute after the process has completed");            
            }
        }
        else
        {
            $line =~ s/\r|\n//g;
            my $lockcommand = join "", "ps ux \| grep ", $line, "\| grep -v grep \| grep sqlTDtoGS.pl";             
            my $lockprocess = `$lockcommand`;
            if ($lockprocess ne "")
            {
                close(FI);    
                core::logger::writeerrorconsoleandlog(__FILE__, __LINE__, "Another DSC process is running on same folder, re-execute after the process has completed");
            }        
        }
        
        # only one line is expected in file
        $line = <FI>;
        if ($line ne "")
        {
            close(FI);    
            core::logger::writeerrorconsoleandlog(__FILE__, __LINE__, "Invalid contents in $lockfilepath, delete the file and re-execute");
        }
        
        close(FI);                
    }
    
    open(FO, ">$lockfilepath") or core::logger::writeerrorconsoleandlog(__FILE__, __LINE__, "Opening file $lockfilepath failed with $!");
	
	if ($systemversion !~ /Microsoft/ && $systemversion !~ /MSW/)
	{ 
		chmod (0600, $lockfilepath) or core::logger::writeerrorconsoleandlog(__FILE__, __LINE__, "changing permissions for file $lockfilepath failed with $!");        
	}
	else
	{ 
		$chmodcommand = sprintf "icacls %s \/grant:r \*%s:(F,WDAC) \/inheritance:r", $lockfilepath, $usersid;
		$chmodcmdresult =`$chmodcommand`;
	}
    
    print FO ($$);
    close(FO);    
    $lockfilecreated = 1;
    return;
}

# Description: Remove the lock file
# Params: NA
# Return: NA or die on failure
sub removelockfile
{
    if ($lockfilecreated == 1)
    {
        unlink($lockfilepath);
        $lockfilecreated = 0;
    }
    
    return;
}


my $OLDERR;

sub restore_streams
{
  close(STDERR) || die "Can't close STDERR: $!";
  open(STDERR, ">&", $OLDERR) || die "Can't restore stderr: $!";
}

sub redirect_streams
{
  open $OLDERR,">&STDERR" || die "Can't duplicate STDERR: $!";
  open(STDERR,">&", core::logger::LOGHANDLE);
}

# Description: Utilities entry function
# Params: NA
# Return: 1 on success
sub main
{
    my $migrationexecoutput;
    my $ret = $core::errorcodes::MIG_SUCCESS;
    
    core::logger::writeinfoconsoleandlog(__FILE__, __LINE__, "Perl file DSC started");
    
    #Check the options specified by user
    $ret = precheck();
    if ($ret != $core::errorcodes::MIG_SUCCESS)
    {
        print "Precheck of DSC failed\n";        
        return $ret;
    }
    
    
    
    #evaluating and setting a timeout
    local $SIG{ALRM} = sub { die "alarm\n" }; 	
    alarm $exectimeout * 60 * 60;
    
    eval 
    {
         #Create lock file to prevent concurrency
         createlockfile();
	
		#pull the sql scripts from the perl scripts
		if (-f $location)
		{
			$totalperlfilecount = 1;
			$ret = readsub($location, 0, \$extractiondirfailcount);
			if ($ret != $core::errorcodes::MIG_SUCCESS)
			{
				return $ret;
			}
					
			$premigrate = 0;
			if($nopl == 1)
			{
				core::logger::writeerrorconsoleandlog(__FILE__, __LINE__, "No PL files found in the input folder. Hence DSC stopped.");
			}
		}
		elsif (-d $location)
		{
			$ret = core::utils::checkdir($location);
			if ($ret != $core::errorcodes::MIG_SUCCESS)
			{
				return $ret;
			}
			
			$ret = readsub($location, 0,  \$extractiondirfailcount);
			if ($ret != $core::errorcodes::MIG_SUCCESS)
			{
				return $ret;
			}    
		
			$allperlfilecount = scalar @fileList;
			
			my $fileCount = 0;
			for my $index (0 .. $#fileList) {
				 my $filesz = -s "$fileList[$index]";
				 if($filesz > $max_file_size)
				 {		
					core::logger::writeinfoconsoleandlog(__FILE__, __LINE__, "[INFO] The file: $fileList[$index] will not be converted as the size of the file is greater than the max configured file size: $appconfig{'MaxFileSize'}\n");
					delete $fileList[$index];
					
				 }
				 else
				 {
					$fileCount = $fileCount + 1;
				 }
				
			}    
			
			
			if ($fileCount == 0)
			{
				core::logger::writeerrorconsoleandlog(__FILE__, __LINE__, "No valid PL files found in the input folder. Hence DSC stopped.");
			}
			
			$totalperlfilecount = $fileCount;
			core::logger::writeinfoconsoleandlog(__FILE__, __LINE__, "Stage 1/3: Extracting SQL contents from perl files.");
			$premigrate = 0;
			$ret = extractsqlcontents();
			if ($ret != $core::errorcodes::MIG_SUCCESS)
			{
				core::logger::writeerrorlogmsg(__FILE__, __LINE__, "Extracting sql contents from perl files failed.");
			}

			core::progress::clearcurrentlineprintlog(__FILE__, __LINE__, "Extraction completed.");                
		}  
			
		redirect_streams();    
		$ret = executemigrationcommand(\$migrationexecoutput);
		restore_streams();
		if ($ret != $core::errorcodes::MIG_SUCCESS)
		{
			return $ret;
		}
		
		core::logger::writeinfoconsoleandlog(__FILE__, __LINE__, "Stage 3/3: Merging SQL contents.");
			
		#Merge the source perl scripts and the generated gauss scripts
		if (-f $location)
		{
			$ret = readsub($location, 1, \$mergedirfailcount);
			if ($ret != $core::errorcodes::MIG_SUCCESS)
			{
				return $ret;
			}
			
		}
		elsif (-d $location)
		{
			$migratedperlfilescount = 0;
			$ret = readsub($location, 1, \$mergedirfailcount);
			if ($ret != $core::errorcodes::MIG_SUCCESS)
			{
				return $ret;
			}
					
			$ret = mergesqlcontents();
			if ($ret != $core::errorcodes::MIG_SUCCESS)
			{
				core::logger::writeerrorlogmsg(__FILE__, __LINE__, "Merging sql contents to perl files failed.");
			}
					
			core::progress::clearcurrentlineprintlog(__FILE__, __LINE__, "Merging completed.");        
		}  
    removelockfile();
    };
    alarm 0;
    
    if($@ =~ m/.*timeout.*/) 
    {
        core::logger::writeerrorconslogret(__FILE__, __LINE__, "DSC took more time than the configured time for execution. Aborting..");
        return $core::errorcodes::COMMAND_EXEC_FAILURE;
    }
    elsif ($@)
    {
        die $@;
    }

    core::progress::printsummary($migrationexecoutput);
    
    return $core::errorcodes::MIG_SUCCESS;
}

# Description: clean up function before exit
# Params: NA
# Return: NA
END 
{
    removelockfile; 
    core::logger::destroylog;
    print "Check log files for more details\n";    
}

return 1;
