#!/usr/bin/perl -w
package core::utils;
use File::stat;
use File::Path;
use File::Basename;
use File::Copy;
use File::Find;
use Fcntl ':mode';

use core::errorcodes;
use strict;
use warnings;

# Description: Check 2 integers
# Params: val1, val2
# Return: 1, -1 or 0
sub compare
{
    my $val1 = shift;
    my $val2 = shift;
    
    if($val1 > $val2) 
    {
        return 1;
    }
    elsif($val2 > $val1) 
    {
        return -1;
    }
    
    return 0;
}

# Description: Check if any content exist for the specified input folder
# Params: input directory
# Return: NA or die on failure
sub checkcontentexist
{
    my ($file_t) = @_;
    if (-f $file_t)
    {
        core::logger::writeerrorconsolepremigratelogret(__FILE__, __LINE__, "$file_t should be a directory");
        return $core::errorcodes::INVALID_ARGUMENT;
    }
    
    if (-d $file_t)
    {
        if (!opendir(SUB, $file_t))
        {
            core::logger::writeerrorconsolepremigratelogret(__FILE__, __LINE__, "opendir of $file_t failed with $!");
            return $core::errorcodes::OPEN_DIR_FAILED;
        }
        
        my @list = readdir(SUB);
        closedir(SUB);    
        
        my $file_to_act;
        foreach $file_to_act (sort @list)
        {
            if ($file_to_act =~ /\.$/)
            {
                next;
            }
            else
            {
                core::logger::writeerrorconsolepremigratelogret(__FILE__, __LINE__, "Contents exist in $file_t, delete the folder and re-run");
                return $core::errorcodes::INVALID_ARGUMENT;
            }    
        }
    }
    
    return $core::errorcodes::MIG_SUCCESS;
}

# Description: get the encoding format of a file
# Params: NA
# Return: NA or die on failure
sub getfileencoding
{
    my $filename = shift;    
    my $encoding = shift;
    my $ret;
    
    my $tempfile = $filename;
    $tempfile =~ s/\.pl$/.pl.temp/;    
    
    if ($core::teradatacore::encodingformat ne '')
    {
        $$encoding = $core::teradatacore::encodingformat;
        return $core::errorcodes::MIG_SUCCESS;
    }
    
    if (defined($core::teradatacore::encodingfiledetails{$filename}) && $core::teradatacore::encodingfiledetails{$filename} ne '')
    {
        $$encoding = $core::teradatacore::encodingfiledetails{$filename};
        return $core::errorcodes::MIG_SUCCESS;
    }    
    
    my $commandoutput  = ''; 
	my $encodingcommandresult;
    
	if ($core::teradatacore::systemversion =~ /Microsoft/ || $core::teradatacore::systemversion =~ /MSW/)
	{
		my $encodingcommand = join "", "java -jar ", $core::teradatacore::migrationtoolpath, $core::teradatacore::directoryseparator, "bin", $core::teradatacore::directoryseparator, "dsctool.jar guessencoding --file-name ", $filename;
		$encodingcommandresult = `$encodingcommand`;
	}
	else
	{
		$encodingcommandresult = `file -pi $filename`;
	}
    
    # check if charset is available for file, if charset is available use it, else remove perl and get the charset
    if ($encodingcommandresult =~ m/charset=/)
    {
        $encodingcommandresult =~ m/.*?charset=(.*)/;
        $$encoding = $1;
    }
    else
    {
		# Remove #!/usr/bin/perl -w and check encoding, this is not required in windows as the buffer contents are already verified
        if ($core::teradatacore::systemversion !~ /Microsoft/ && $core::teradatacore::systemversion !~ /MSW/)
        {
			my $statfile = stat($filename);
			my $val = $statfile->mode & 0400;
			if ($val == 0)
			{
				my $diemsg = sprintf("File $filename does not have read permissions.");
				core::logger::writeerrorconsolelogret(__FILE__, __LINE__, $diemsg);
				return $core::errorcodes::INVALID_PERMISSIONS;
			}
			
			my $basefilepath = dirname($filename);
			$statfile = stat($basefilepath);
			$val = $statfile->mode & 0500;
			if ($val == 0)                
			{
				print "Directory $basefilepath does not have read/execute permissions.";
				return $core::errorcodes::INVALID_PERMISSIONS;
			}
    
            # Create the temporary file excluding /usr/bin/perl.
            # If file has this content we will not be able to get the correct encoding format
            $ret = execcommand("grep -v '#\!/usr/bin/perl' $filename > $tempfile", \$commandoutput, 0);          
            if ($ret != $core::errorcodes::MIG_SUCCESS)
            {
                return $ret;
            }
			
			$$encoding = `file -pi $tempfile | cut -d '=' -f 2`;
        }
    }
    
    my $unknowntext = 'unknown';
    my $binarytext = 'binary';
    
    # Some files may have binary or unknown encoding format, in this case get the locale charset
    if ($$encoding =~ m/\Q$unknowntext/ or $$encoding =~ m/\Q$binarytext/)
    {
        core::logger::writeinfologmsg(__FILE__, __LINE__, "Unable to identify the charset for file: $filename, system locale charset will be used for processing");
        $$encoding = `locale charmap`;
    }
    
    chomp($$encoding);    
    core::logger::writeinfologmsg(__FILE__, __LINE__, "File $filename will be read with charset: $$encoding");
    
    # Store this information in shared map
    # Same format can be used while creating the output file with the migrated sql contents
    $core::teradatacore::encodingfiledetails{$filename} = $$encoding;
    unlink($tempfile);        
    return $core::errorcodes::MIG_SUCCESS;
}

# Description: check if file is valid
# Params: file path
# Return: NA or die on failure
sub checkfile
{
    my($location) = @_;

    if ($core::teradatacore::systemversion =~ /Microsoft/ || $core::teradatacore::systemversion =~ /MSW/)
    {
        return $core::errorcodes::MIG_SUCCESS;
    }
    
    my $statfile = stat($location);
    my $val = $statfile->mode & 00022;
    if ($val gt 0)
    {
        my $diemsg = sprintf("Migration Application failed to start : Since the input file $location  has write privileges to Group and/or Others, process is stopped due to security reason.");
        core::logger::writeerrorconsolepremigratelogret(__FILE__, __LINE__, $diemsg);
        return $core::errorcodes::INVALID_PERMISSIONS;
    }
    
    return $core::errorcodes::MIG_SUCCESS;
}

# Description: check if directory is valid
# Params: dir path
# Return: NA or die on failure
sub checkdir
{
    my($location) = @_;
    
    if ($core::teradatacore::systemversion =~ /Microsoft/ || $core::teradatacore::systemversion =~ /MSW/)
    {
        return $core::errorcodes::MIG_SUCCESS;
    }
    
    my $statfile = stat($location);
    my $val = $statfile->mode & 00022;
    if ($val gt 0)
    {
        my $diemsg = sprintf("Directory $location  has write privileges to Group and/or Others.");
        core::logger::writeerrorconsolepremigratelogret(__FILE__, __LINE__, $diemsg);        
        return $core::errorcodes::INVALID_PERMISSIONS;
    }

    $val = $statfile->mode & 0100;
    if ($val == 0)                
    {
        my $diemsg = sprintf("Directory $location does not have execute permissions.");
        core::logger::writeerrorconsolepremigratelogret(__FILE__, __LINE__, $diemsg);        
        return $core::errorcodes::INVALID_PERMISSIONS;
    }    
    return $core::errorcodes::MIG_SUCCESS;
}

# Description: Wrapper function for execution of any command
# Params: The command that should be executed
# Return: NA or die on failure
sub execcommand
{   
    my $command = shift;
    my $output = shift;
    my $printoutput = shift;
    my $diemsg;

    $core::teradatacore::commandexecutioncompleted = 0;
        
    $$output = `$command 2>&1`;    
    $core::teradatacore::commandexecutioncompleted = 1;
    if ($? == -1)
    {
        $diemsg = sprintf("failed to execute: $!");
        core::logger::writeerrorlogmsg(__FILE__, __LINE__, $diemsg);
        return $core::errorcodes::COMMAND_EXEC_FAILURE;
    }
    elsif ($? & 127)
    {
        $diemsg = sprintf("child died with signal %d, %s coredump",  ($? & 127),  ($? & 128) ? 'with' : 'without');
        core::logger::writeerrorlogmsg(__FILE__, __LINE__, $diemsg);
        return $core::errorcodes::COMMAND_EXEC_FAILURE;
    }
    elsif (($? >> 8) ne 0)
    {
        $diemsg = sprintf("Execution of command $command failed");
        core::logger::writeerrorlogmsg(__FILE__, __LINE__, $diemsg);
        return $core::errorcodes::COMMAND_EXEC_FAILURE;
    }
    
    if ($printoutput)
    {
        if ($$output ne '')
        {
            core::logger::writeinfologmsg(__FILE__, __LINE__, "Command $command execution output: \n$$output");
        }
        else
        {
            core::logger::writeinfologmsg(__FILE__, __LINE__, "Command $command executed successfully");
        }
    }
    
    return $core::errorcodes::MIG_SUCCESS;
}

return 1;